@@ -36,7 +36,7 @@ static char *preptree(char *, int *, struct noeud **pracine, struct noeud ***pta
 static int loadphylip(char *arbre, int *prooted, struct noeud **, branche *);
 struct noeud *unrootedset(char *deb, char *fin, branche **p_int_br, struct noeud **tabtax,
 	branche *);
-char *recur_ecrit_arbre(struct noeud *centre, struct noeud *pere, char *arbre, char *finarbre,
+static char *recur_write_tree(struct noeud *centre, struct noeud *pere, char *arbre, char *finarbre,
 	branche *);
 static int get_br_from_bouts(struct noeud *a, struct noeud *b, branche *);
 static int for_all_branches(struct noeud *centre, struct noeud *origine, branche *branches);
@@ -60,7 +60,7 @@ void make_strict_consensus(char *tree);
 
 /*  extern functions */
 extern char *nextpar(char *pospar);
-extern double place_midpoint_root(struct noeud *from, struct noeud *racine);
+extern double place_midpoint_root(struct noeud *from, struct noeud *racine, int fixed_outgroup);
 extern const char *make_binary_or_unrooted(char *arbre);
 
 
@@ -90,7 +90,7 @@ int init_add_bootstrap(char *full_tree)
 	//to have same tree display as without bootstrap, must root it at midpoint and go over branches
 	//from there
 	struct noeud *newroot = (struct noeud *)malloc(sizeof(struct noeud));//deallocated later by free_arbre
-	place_midpoint_root(racine, newroot); // root at midpoint
+	place_midpoint_root(racine, newroot, FALSE); // root at midpoint
 	newroot->nom = NULL;
 	struct noeud *v1 = newroot->v1; struct noeud *v2 = newroot->v2;
 	//remove root
@@ -169,13 +169,13 @@ char *finish_add_bootstrap(int replicates)
 	arbre1 = (char *)malloc( taille);
 	arbre2 = (char *)malloc( taille);
 	arbre3 = (char *)malloc( taille);
-	f1 = recur_ecrit_arbre(r1, r1p, arbre1, arbre1 + taille, mainbranches);
+	f1 = recur_write_tree(r1, r1p, arbre1, arbre1 + taille, mainbranches);
 	if(f1 == NULL) return NULL;
 	*(f1 + 1) = 0;
-	f2 = recur_ecrit_arbre(r2, r2p, arbre2, arbre2 + taille, mainbranches);
+	f2 = recur_write_tree(r2, r2p, arbre2, arbre2 + taille, mainbranches);
 	if(f2 == NULL) return NULL;
 	*(f2 + 1) = 0;
-	f3 = recur_ecrit_arbre(r3, r3p, arbre3, arbre3 + taille, mainbranches);
+	f3 = recur_write_tree(r3, r3p, arbre3, arbre3 + taille, mainbranches);
 	if(f3 == NULL) return NULL;
 	*(f3 + 1) = 0;
 	arbre = (char *)malloc( f1 - arbre1 + f2 - arbre2 + f3 - arbre3 + 100);
@@ -221,7 +221,7 @@ int i, c, rooted, v, error;
 char *arbre, *p;
 double root_br_l;
 struct noeud *racine, *fils, **tabtax;
-branche *branches;
+branche *branches = NULL;
 
 	arbre = strdup(tree);
 	/* lecture de l'arbre */
@@ -433,7 +433,7 @@ if(virg != NULL && virg < fin) { /* traitement longueur */
 else	{
 	brlength = 1;
 	has_br_length=0;
-	virg = fin + 1;
+	//virg = fin + 1;
 	}
 /* recherche bootstrap (internal label) */
 if(totbranches < notu-3) {
@@ -446,7 +446,7 @@ return p;
 
 
 /* calcul d'un arbre racine au format phylip */
-char *recur_ecrit_arbre(struct noeud *centre, struct noeud *pere, char *arbre, char *finarbre,
+static char *recur_write_tree(struct noeud *centre, struct noeud *pere, char *arbre, char *finarbre,
 	branche *branches)
 //returns pter to last used byte of tree or NULL iff error
 //caution: no null byte after last used byte !
@@ -474,7 +474,7 @@ else	{
 		}
 	*arbre='(';
 	p = arbre;
-	arbre=recur_ecrit_arbre(gauche, centre, arbre+1, finarbre, branches);
+	arbre=recur_write_tree(gauche, centre, arbre+1, finarbre, branches);
 	if(arbre==NULL) return NULL;
 	if(has_br_length) {
 		if(arbre+10>=finarbre) return NULL;
@@ -483,7 +483,7 @@ else	{
 		}
 	else arbre++;
 	*arbre=',';
-	arbre = recur_ecrit_arbre(droit, centre, arbre+1, finarbre, branches);
+	arbre = recur_write_tree(droit, centre, arbre+1, finarbre, branches);
 	if(arbre==NULL) return NULL;
 	if(has_br_length) {
 		if(arbre+10>=finarbre) return NULL;
@@ -789,5 +789,5 @@ to produce a strict consensus tree
 				}
 			}
 		}
-	p = tree;
+	//p = tree;
 }
