@@ -17,16 +17,15 @@ int save_part_as_pir(int debut, int fin, char **sequence, char **seqname,
 	int *sel_seqs, int tot_seqs, char *fname, int *withU, int *num_longest, int protein);
 int replace_align_part(SEA_VIEW *view, int debut, int fin, char *fname, int withU, int num_longest,
 	int *tmpselseqs, align_choice mychoice, const char **err_message);
-void align_selected_parts(SEA_VIEW *view, int align_algorithm, align_choice my_align_choice);
 int calc_gap_sites(char *old_seq, char *new_seq, int lold, int lnew, 
 	gap_site *gap_sites, int maxsites);
 int insert_gaps_new_align(char **seq, int site, int number, int numseqs, 
 	int lseqs);
 int reset_stars(SEA_VIEW *view, int debut, int lpart, char **seq, int lfrag, int *tmpselseqs);
 int confirm_refer_seq(int num_longest, SEA_VIEW *view, int *tmpselseqs);
-char *calc_msa_command(SEA_VIEW *view, char *base_fname);
+char *calc_msa_command(SEA_VIEW *view, char *base_fname, void (*alert)(const char *,...) );
 void cre_align_menu(SEA_VIEW *view);
-void align_opt_callback(Fl_Widget *ob, void *data);
+void align_opt_callback(Fl_Widget *ob, long unused);
 char *get_algo_opts(int algonum /* from 0 */, char **presname);
 char *get_opt_resname(int algonum);
 void set_algo_option_item(char *opts, Fl_Menu_Item *menu, int itemrank, int checkit);
@@ -42,8 +41,10 @@ int save_fasta_file_no_gaponly(char *fname, char **sequence, int tot_seqs, int *
 void unalign(SEA_VIEW *view);
 int reorder_seqs(int tot, char **seq, char **name);
 int check_alignment_characters(SEA_VIEW *view, int *sel_seqs);
+int command_line_align(int argc, char **argv);
 
 /* extern proto */
+extern const char *progname;
 extern int run_external_prog_in_pseudoterm(char *cmd, const char *dialogfname, const char *label);
 extern int insert_gaps_at(SEA_VIEW *view, int seq, int site, int total);
 extern int insert_gap_all_comments(int numgaps, int pos,  SEA_VIEW *view);
@@ -59,6 +60,21 @@ extern void direct_help_callback(Fl_Widget *wgt, void *data);
 extern int compute_size_params(SEA_VIEW *view, int force_recompute);
 extern char **prepcolranks_by_codon(char **seq, int totseqs, int maxlen, int *eachlength, char **comments);
 extern int is_view_valid(SEA_VIEW *view);
+extern int calc_max_seq_length(int seq_length, int tot_seqs);
+extern void allonge_seqs(char **seq, int totseqs, int maxlen, int *eachlength,
+			 int tot_comment_lines, char **comment_line, char **pregion_line);
+extern SEA_VIEW *cmdline_read_input_alignment(int argc, char **argv);
+extern bool isarg(int argc, char *argv[], const char *arg);
+extern float argval(int argc, char *argv[], const char *arg, float defval);
+extern char *argname(int argc, char *argv[], const char *arg);
+extern int get_ncbi_gc_from_comment(char *comment);
+extern char *translate_with_gaps(char *seq, int gc);
+extern char *back_translate_with_gaps(char *prot, char *dna);
+extern char *process_output_options(int argc, char **argv, known_format& out_format, bool& std_output);
+extern void load_resources(const char *progname);
+extern "C" {
+  int get_acnuc_gc_number(int ncbi_gc);
+}
 
 
 /* extern variables */
@@ -132,23 +148,24 @@ if(l == 0) *(int *)data = TRUE;
 #endif
 
 
-void align_selected_parts(SEA_VIEW *view, int align_algorithm, align_choice my_align_choice)
+void align_selected_parts(SEA_VIEW *view, align_choice my_align_choice, bool no_gui)
 {
 int debut, fin, status, withU, *tmpselseqs, num_longest, l;
 char *p;
-char base_fname[150], fname[150];
+char base_fname[PATH_MAX], fname[PATH_MAX];
 char *commande;
 FILE *in;
+  void (*alert)(const char *,...) = no_gui ? Fl::fatal : fl_alert;
 
 if( my_align_choice == ALIGN_SITES && (view->active_region == NULL || view->active_region->list == NULL ||
 	view->active_region->list->next != NULL || view->tot_sel_seqs <= 1) ) {
-	fl_message("Need to have exactly one block of selected sites\n"
+	alert("Need to have exactly one block of selected sites\n"
 		"and some selected sequences");
 	return;
 	}
 if(my_align_choice == ALIGN_SELECTED_SEQS) {
 	if(view->tot_sel_seqs == 0) return;
-	if( ! fl_choice("Confirm alignment of %d selected sequences ?\n"
+	if ( !no_gui && ! fl_choice("Confirm alignment of %d selected sequences ?\n"
 					"(Any previous alignment between selected and other sequences will disappear)", 
 					"Cancel", "Align", NULL, view->tot_sel_seqs) ) return;
 	}
@@ -163,10 +180,6 @@ else	{
 p = create_tmp_filename();
 if(p == NULL) return;
 strcpy(base_fname, p);
-/* 
-convertir en minuscules car clustalw n'accepte pas les majuscules ds filenames
-*/
-p = base_fname - 1; while( *(++p) != 0) *p = tolower( *p);
 sprintf(fname, "%s.pir", base_fname);
 /* allonger les seqs plus courtes que la region traitee sans compter comme modif alignment*/
 int old_modif_but_not_saved = view->modif_but_not_saved;
@@ -175,7 +188,7 @@ for(l = 0; l < view->tot_seqs; l++) {
 	if(fin > view->each_length[l]) insert_gaps_at(view, l + 1, 
 			view->each_length[l] + 1, fin - view->each_length[l]);
 	}
-set_seaview_modified(view, old_modif_but_not_saved);
+if (!no_gui) set_seaview_modified(view, old_modif_but_not_saved);
 /* prepare list of empty seqs */
 tmpselseqs = (int *)malloc(view->tot_seqs * sizeof(int));
 if(my_align_choice == ALIGN_ALL)
@@ -188,28 +201,33 @@ status = save_part_as_pir(debut, fin, view->sequence, view->seqname,
 //if(view->protein) //clustalw2 now accepts U and muscle also
 	withU = FALSE;
 if(status) {
-	fl_alert("Cannot write sequences to filename\n%s", fname);
+	alert("Cannot write sequences to filename\n%s", fname);
 	free(tmpselseqs);
 	delete_tmp_filename(base_fname);
 	return;
 	}
-if( my_align_choice == ALIGN_SITES ) num_longest = confirm_refer_seq(num_longest, view, tmpselseqs);
+if( !no_gui && my_align_choice == ALIGN_SITES ) num_longest = confirm_refer_seq(num_longest, view, tmpselseqs);
 if(num_longest == -1 ) {
-	sprintf(fname, "%s.pir", base_fname);
-	remove(fname);
 	free(tmpselseqs);
 	delete_tmp_filename(base_fname);
 	return;
 	}
 
-commande = calc_msa_command(view, base_fname);
+commande = calc_msa_command(view, base_fname, alert);
 status = (commande == NULL);
 if(status) {
 	free(tmpselseqs);
 	delete_tmp_filename(base_fname);
 	return;
 	}
-status = run_external_prog_in_pseudoterm(commande, NULL, "alignment");
+  if (no_gui) {
+#ifdef WIN32
+#define system(a) mysystem(a)
+    extern int mysystem(const char *command);
+#endif
+    status = system(commande);
+    }
+  else status = run_external_prog_in_pseudoterm(commande, NULL, "alignment");
 free(commande);
 sprintf(fname, "%s.out", base_fname);
 if( status == 0 ) {
@@ -221,14 +239,14 @@ if( status == 0 ) {
 		}
 	else	status = 1;
 	}
-if (is_view_valid(view)) {
+if (no_gui || is_view_valid(view)) {
   if( status != 0 )
-	  fl_message("%s: alignment operation ignored", extract_filename(view->masename));
+	  alert("%s: alignment operation ignored", extract_filename(view->masename));
   else	{
 	  const char *err_message;
 	  status = replace_align_part(view, debut, fin, fname, withU, num_longest, tmpselseqs, 
 				      my_align_choice, &err_message );
-	  if (status) fl_alert("%s", err_message);
+	  if (status) alert("%s", err_message);
 	  else if(my_align_choice == ALIGN_ALL) del_gap_only_sites(view);
 	  }
 }
@@ -404,41 +422,53 @@ if(mychoice == ALIGN_SITES) {
 		if(view->numb_gc == 1) continue;
 		for(i = debut - 1; i < debut + l_copy - 1; i++) {
 			res = view->sequence[num][i];
-			view->col_rank[num][i] = (char)calc_color_function( res );
+			if (view->col_rank) view->col_rank[num][i] = (char)calc_color_function( res );
 			}
 		}
 	}
 else {
-	newlength = view->seq_length;
+  if (lfrag > view->max_seq_length) {
+    view->max_seq_length = calc_max_seq_length(lfrag, view->tot_seqs);
+    for (num = 0; view->col_rank && num < view->tot_seqs; num++) {
+      free(view->col_rank[num]);
+      }
+    if (view->col_rank) free(view->col_rank);
+    allonge_seqs(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
+		 view->tot_comment_lines, view->comment_line, &view->region_line);
+    if (view->col_rank) view->col_rank = prepcolranks(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
+				  ( view->protein ? get_color_for_aa : get_color_for_base ), 
+				  view->numb_gc, view->allow_lower);
+  }	   
+  newlength = view->seq_length;
 	rang = -1;
 	for(num = 0; num < view->tot_seqs; num++) {
 		if(  ! tmpselseqs[num] ) continue;
 		rang++;
 		/* on met les nouvelles seqs */
 		l_copy = strlen(seq[rang]);
-		if(l_copy > view->max_seq_length) { l_copy = view->max_seq_length; seq[rang][l_copy] = 0; }
 		strcpy(view->sequence[num], seq[rang]);
 		view->each_length[num] = l_copy;
 		newlength = FL_max(newlength, l_copy);
 		if(view->numb_gc > 1) {
-			for(i = 0; i < view->each_length[num]; i++) {
+			for (i = 0; view->col_rank && i < view->each_length[num]; i++) {
 				res = view->sequence[num][i];
 				view->col_rank[num][i] = (char)calc_color_function( res );
 				}
 			}
 	}
 }
-/* mettre a jour horsli */
-if(mychoice == ALIGN_ALL) {
-	view->seq_length = 0;
-	for(num = 0; num < view->tot_seqs; num++) view->seq_length = FL_max(view->seq_length, view->each_length[num]);
-	compute_size_params(view, TRUE);
-	view->horsli->redraw();
-	}
-else update_current_seq_length(newlength, view);
-set_seaview_modified(view, TRUE);
-retval = FALSE;
-
+  if (view->col_rank) {
+    /* mettre a jour horsli */
+    if(mychoice == ALIGN_ALL) {
+	    view->seq_length = 0;
+	    for(num = 0; num < view->tot_seqs; num++) view->seq_length = FL_max(view->seq_length, view->each_length[num]);
+	    compute_size_params(view, TRUE);
+	    view->horsli->redraw();
+	    }
+    else update_current_seq_length(newlength, view);
+    set_seaview_modified(view, TRUE);
+  }
+  retval = FALSE;
 fin:
 for(num = 0; num < total; num++) 
 	free(seq[num]);
@@ -529,7 +559,7 @@ form->show();
 for (;;) {
 	Fl_Widget *o = Fl::readqueue();
 	if (!o) Fl::wait();
-	else if(o == cancel_button ) { num_longest = -1; break; }
+	else if (o == cancel_button || o == form) { num_longest = -1; break; }
 	else if(o == ok_button && browser_noms->value() != 0) break;
 	}
 form->hide();
@@ -548,7 +578,7 @@ return num_longest;
 }
 
 
-char *calc_msa_command(SEA_VIEW *view, char *base_fname)
+char *calc_msa_command(SEA_VIEW *view, char *base_fname, void (*alert)(const char *,...) )
 {
 char *command;
 char *algo_name;
@@ -570,7 +600,7 @@ status = (p == NULL);
 status = (get_full_path(algo_name) == NULL);
 #endif
 if(status) {
-	fl_alert("Alignment is impossible because program  '%s'  is not found\n"
+	alert("Alignment is impossible because program  '%s'  is not found\n"
 	"in PATH or seaview's directory", algo_name);
 	free(algo_name);
 	return NULL;
@@ -584,10 +614,9 @@ int clustalopt = ALIGN_OPTIONS + 1;
 optargs = get_algo_opts(view->alignment_algorithm, NULL);
 status = view->menu_align[clustalopt + MAX_MSA_ALGOS ].flags;
 if(optargs != NULL && (status & FL_MENU_VALUE) ) {
-	if((p=strchr(local, '>')) != NULL) {
-		*p = 0;
-		sprintf(tmp, "%s %s > %s", local, optargs, p + 1);
-		}
+	if (strchr(local, '>') != NULL) {
+	  sprintf(tmp, "%s %s", optargs, local);
+	  }
 	else {
 		sprintf(tmp, "%s %s", local, optargs);
 		}
@@ -601,10 +630,24 @@ if(optargs != NULL && (status & FL_MENU_VALUE) ) {
   
 /* replace all %f by file names */
 while((p = strstr(local, "%f")) != NULL) {
-	*p = 0;
-	sprintf(tmp, "%s%s%s", local, base_fname, p+2);
-	strcpy(local, tmp);
-	}
+  char *q, *r, s;
+  *p = 0;
+  q = p;
+  do q--; while (q > local && *q != ' ');
+  q++;
+  r = p;
+  do r++; while (*r != ' ' && *r != 0);
+  s = *q;
+  *q = 0;
+  strcpy(tmp, local);
+  *q = s;
+  s = *r;
+  *r = 0;
+  sprintf(tmp + strlen(tmp), "\"%s%s%s\"", q, base_fname, p+2);
+  *r = s;
+  if (*r) strcat(tmp, r);
+  strcpy(local, tmp);
+  }
 /* build complete command string */
 #if defined(WIN32)
 	command = (char *)malloc(strlen(w32path) + strlen(local) + 30);
@@ -623,10 +666,10 @@ extern void fix_paste_timeout(void *u);
 void align_menu_callback(Fl_Widget *ob, void *data)
 {
   SEA_VIEW *view = (SEA_VIEW *) ob->user_data();
-	int reponse = view->menubar->mvalue() - view->menu_align;
+  int reponse = ((Fl_Menu_*)ob)->mvalue() - view->menu_align;
 	Fl::remove_timeout(fix_paste_timeout);
 	if(reponse <= ALIGN_SITES) { 
-	  align_selected_parts(view, view->alignment_algorithm, (align_choice)reponse );
+	  align_selected_parts(view, (align_choice)reponse );
 	  if(is_view_valid(view)) {
 	    view->DNA_obj->redraw();
 	    view->horsli->redraw();
@@ -674,7 +717,7 @@ void cre_align_menu(SEA_VIEW *view)
     {0}
   };
   
-  view->alignment_algorithm = int_res_value("alignment", 1);
+  view->alignment_algorithm = int_res_value("alignment", 0);
   view->count_msa_algos = int_res_value("msa_algo_count", 0);
 
   Fl_Menu_Item* alignitems = new Fl_Menu_Item[sizeof(alignitems_static)/sizeof(Fl_Menu_Item) + MAX_MSA_ALGOS];
@@ -684,7 +727,7 @@ void cre_align_menu(SEA_VIEW *view)
 
   for(elt = 1; elt <= MAX_MSA_ALGOS; elt++) {
     attr = FL_MENU_RADIO; 
-    alignitems[clustalopt + elt - 1].callback(align_opt_callback, (void *)(elt - 1));
+    alignitems[clustalopt + elt - 1].callback(align_opt_callback, elt - 1);
     if (elt <= view->count_msa_algos) {
       sprintf(msa_name, "msa_name_%d", elt);
       name = get_res_value(msa_name, NULL);
@@ -913,13 +956,13 @@ save_resources();
 }
 
 
-void align_opt_callback(Fl_Widget *ob, void *data)
+void align_opt_callback(Fl_Widget *ob, long unused)
 {
 SEA_VIEW *view = (SEA_VIEW *)ob->user_data();
 char *opts;
 int clustalopt = ALIGN_OPTIONS + 1;
 
-int reponse =  view->menubar->mvalue() - view->menu_align - clustalopt;
+int reponse =  ((Fl_Menu_*)ob)->mvalue() - view->menu_align - clustalopt;
 if(reponse >= 0 && reponse <  view->count_msa_algos) { 
 	view->alignment_algorithm = reponse;  
 	opts = get_algo_opts(view->alignment_algorithm, NULL);
@@ -961,14 +1004,14 @@ else if(checkit) status |= FL_MENU_VALUE;
 
 void init_msa_algos(void)
 {
-int count = int_res_value("msa_algo_count", 0);
-set_res_value("msa_name_1", "clustalo"); 
-set_res_value("msa_args_1", "-v -i %f.pir -o %f.out --outfmt=fasta");
-if(count < 2) {
-  set_res_value("msa_algo_count", "2");
-  set_res_value("alignment", "0"); /* clustal-omega is default starting msa method */
+  int count = int_res_value("msa_algo_count", 0);
+  set_res_value("msa_name_1", "clustalo"); 
+  set_res_value("msa_args_1", "-v -i %f.pir -o %f.out --outfmt=fasta");
   set_res_value("msa_name_2", "muscle"); 
   set_res_value("msa_args_2", "-in %f.pir -out %f.out");
+  if (count < 2) {
+    set_res_value("msa_algo_count", "2");
+    set_res_value("alignment", "0"); /* clustal-omega is default starting msa method */
   }
 }
 
@@ -1116,29 +1159,22 @@ void profile_align(SEA_VIEW *view)
 	else {
 		command = new char[1000];
 		if(profile_algo == profile_clustalo) {
-			sprintf(command, "\"%s\" -v --profile1=%s -%s%s  --outfmt=fasta "
-				" -o %s.aligned -t %s", p, 
-				fname1, 
-				profile_to_profile ? "-profile2=" : "i ",
+			sprintf(command, 
+				"\"%s\" -v \"--profile1=%s\" %s%s\"  --outfmt=fasta -o \"%s.aligned\" -t %s", 
+				p, fname1, 
+				profile_to_profile ? "\"--profile2=" : "-i \"",
 				fname2, 
 				base_fname,
 				view->protein ? "protein" : "dna" );
 			}
 		else { // muscle
-			sprintf(command, "\"%s\" -profile -in1 %s -in2 %s -out %s.aligned", 
+			sprintf(command, 
+				"\"%s\" -profile -in1 \"%s\" -in2 \"%s\" -out \"%s.aligned\" ", 
 				p, fname1, fname2, base_fname );
 			}
 		status = run_external_prog_in_pseudoterm(command, NULL, "profile alignment");
-		delete command;
+		delete[] command;
 		}
-	remove(fname1);
-	remove(fname2);
-	if(profile_to_profile) {
-		sprintf(fname1, "%s_1.dnd", base_fname);
-		remove(fname1);
-		}
-	sprintf(fname1, "%s_2.dnd", base_fname);
-	remove(fname1);
 	sprintf(fname1, "%s.aligned", base_fname);
 	/* lecture de l'alignement multiple */
 	if( status == 0 ) {
@@ -1444,3 +1480,94 @@ int check_alignment_characters(SEA_VIEW *view, int *sel_seqs)
   return FALSE;
 }
 
+
+int command_line_align(int argc, char **argv)
+{
+  char *p;
+  int i;
+  bool std_output = false;
+  bool need_back_translate = false;
+  char *old_opt_args = NULL;
+  char opt_name[20];
+  SEA_VIEW *DNAview;
+  SEA_VIEW *view = cmdline_read_input_alignment(argc, argv);
+  known_format out_format = view->format_for_save;
+  load_resources(progname);
+  init_msa_algos();
+  view->alignment_algorithm = (int)argval(argc, argv, "-align_algo", int_res_value("alignment", 0));
+  view->menu_align = new Fl_Menu_Item[ALIGN_OPTIONS + MAX_MSA_ALGOS + 2];
+  Fl_Menu_Item *item = (Fl_Menu_Item *)&view->menu_align[ALIGN_OPTIONS + MAX_MSA_ALGOS + 1];
+  item->clear() ;
+  if ( (p = argname(argc, argv, "-align_extra_args")) != NULL) {
+    item->set();
+    sprintf(opt_name, "msa_opt_args_%d", view->alignment_algorithm + 1);
+    old_opt_args = strdup(get_res_value(opt_name, ""));
+    set_res_value(opt_name, p);
+    }
+  
+  if ( !view->protein && isarg(argc, argv, "-align_at_protein_level")) {
+    need_back_translate = true;
+    DNAview = view;
+    view = new SEA_VIEW;
+    memcpy(view, DNAview, sizeof(SEA_VIEW));
+    view->seq_length = 0;
+    view->sequence = (char **)malloc(view->tot_seqs * sizeof(char*));
+    for (i = 0; i < view->tot_seqs; i++) {
+      int gc = (view->comments != NULL ? get_ncbi_gc_from_comment(DNAview->comments[i]) : 1);
+      p = translate_with_gaps(DNAview->sequence[i], get_acnuc_gc_number(gc));
+      view->sequence[i] = p;
+      view->each_length[i] = strlen(p);
+      if (view->each_length[i] > view->seq_length) view->seq_length = view->each_length[i];
+      view->regions = NULL;
+    }
+    view->protein = true;
+    view->max_seq_length = calc_max_seq_length(view->seq_length, view->tot_seqs);
+    allonge_seqs(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
+		 view->tot_comment_lines, view->comment_line, &view->region_line);
+  }
+  
+  align_selected_parts(view, ALIGN_ALL, true);
+
+  if (old_opt_args) set_res_value(opt_name, old_opt_args);
+  if (need_back_translate) {
+    DNAview->seq_length = 0;
+    for (i = 0; i < DNAview->tot_seqs; i++) {
+      p = back_translate_with_gaps(view->sequence[i], DNAview->sequence[i]);
+      free(DNAview->sequence[i]);
+      DNAview->sequence[i] = p;
+      int l = strlen(p);
+      DNAview->each_length[i] = l;
+      if (l > DNAview->seq_length) DNAview->seq_length = l;
+    }
+    view = DNAview;
+  }
+  
+  char *outfile = process_output_options(argc, argv, out_format, std_output);
+
+  char *err_message = save_alignment_or_region(outfile, view->sequence, view->comments,
+					       view->header, view->seqname, view->tot_seqs, view->each_length,
+					       view->regions, NULL, out_format,
+					       view->numb_species_sets, view->list_species_sets,
+					       view->name_species_sets, NULL, 0, view->protein,
+					       view->tot_comment_lines, view->comment_name, 
+					       view->comment_line, view->phylipwidnames,
+					       view->tot_trees, view->trees, 
+					       view->menu_trees->vitem(0), view->spaces_in_fasta_names);
+  if (err_message) {
+    fprintf(stderr, "%s\n", err_message);
+    if (std_output) delete_tmp_filename(outfile);
+    exit(1);
+  }
+  if (std_output) {
+    char line[200];
+    FILE *in = fopen(outfile, "r");
+    while (fgets(line, sizeof(line), in) != NULL) {
+      fputs(line, stdout);
+    }
+    fclose(in);
+    fflush(stdout);
+    delete_tmp_filename(outfile);
+  }
+  return 0;
+}
+
