@@ -12,9 +12,6 @@ typedef int NSInteger;
 typedef unsigned int NSUInteger;
 #endif
 #endif
-#ifndef kCGColorSpaceUserRGB // needed from 10.6
-#define kCGColorSpaceUserRGB  CFSTR("kCGColorSpaceUserRGB")
-#endif
 #include "seaview.h"
 #include "pdf_or_ps.h"
 #include <FL/x.H>
@@ -28,7 +25,8 @@ typedef unsigned int NSUInteger;
 // included functions
 char *mac_fname_to_roman(const char *in);
 const char *MG_GetBundleResourcesDir(void);
-void show_apropos(Fl_Widget *, void *unused);
+static void file_receive_cb(const char *fname);
+static void show_apropos(Fl_Widget *, void *unused);
 void MG_apple_inits(void);
 void set_seaview_modified(SEA_VIEW *view, int ismodified);
 void windowmenuitem_callback(Fl_Widget *o, void *data);
@@ -45,19 +43,20 @@ extern int printout_block, printout_fontsize;
 extern int printout_vary, printout_black;
 extern Fl_Paged_Device::Page_Format printout_pageformat;
 extern Fl_Paged_Device::Page_Layout printout_layout;
+extern Fl_Window *use_initial_file(SEA_VIEW *view, char *masename, int doing_dnd);
 
 /* globals */
 
 char *mac_fname_to_roman(const char *in)
-/* passage codage pathname vers codage MacRoman qui semble necessaire pour display ecran 
+/* passage codage pathname vers codage MacRoman qui est necessaire pour PDFlib/PostScript 
  */
 {
   static char out[250];
   CFStringRef mycfs;
   Boolean ok;
-  /* j'ai cru comprendre que les pathnames sont codes en UTF8 */
+  /* les pathnames sont codes en UTF8 */
   mycfs = CFStringCreateWithCString(NULL, in, kCFStringEncodingUTF8);
-  /* et que MacRoman est utilise pour display */
+  /* conversion vers MacRoman */
   ok = CFStringGetCString(mycfs, out, sizeof(out), kCFStringEncodingMacRoman);
   CFRelease(mycfs);
   return (ok ? out : (char *)in);
@@ -68,14 +67,7 @@ const char *MG_GetBundleResourcesDir(void)
 {
   static char *path = NULL;
   if (!path) {
-    path = (char*)malloc(FL_PATH_MAX);
-    CFURLRef url = CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
-    CFURLGetFileSystemRepresentation(url,
-				   true,
-				   (UInt8*)path,
-				   FL_PATH_MAX);
-    CFRelease(url);
-    path = (char*)realloc(path, strlen(path) + 1);
+    path = strdup([[[NSBundle mainBundle] resourcePath] UTF8String]);
     }
   return path;
 }
@@ -83,7 +75,7 @@ const char *MG_GetBundleResourcesDir(void)
 
 static void label(int x, int y, int w, int h, const char *text, NSView *view)
 {
-  NSRect rect = {x, y, w, h};
+  NSRect rect = {{x, y}, {w, h}};
   NSTextField *label = [[[NSTextField alloc] initWithFrame:rect] autorelease];
   [label setStringValue:[[[NSString alloc] initWithUTF8String:text] autorelease]];
   [label setEditable:NO];
@@ -105,66 +97,67 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
   NSString *dir = [[NSString alloc] initWithUTF8String:directory];
   NSString *preset = [[NSString alloc] initWithUTF8String:dfault];
   NSString *fname = [preset lastPathComponent];
-  NSRect rectview = {0, 0, 500, 65 };
+  NSRect rectview = {{0, 0}, {500, 65} };
   NSView *view = [[[NSView alloc] initWithFrame:rectview] autorelease];
 //blocksize  
   label(12, 15, 45, 30, "block\nsize:", view);
-  NSRect rectblock = {50, 20, 35, 20 };
+  NSRect rectblock = {{50, 20}, {35, 20} };
   NSTextField *blockview = [[[NSTextField alloc] initWithFrame:rectblock] autorelease];
   [view addSubview:blockview];
   [blockview setIntValue:printout_block];
 //fontsize
   label(102, 15, 45, 30, "font\nsize:", view);
-  NSRect rectfont = {140, 20, 35, 20 };
+  NSRect rectfont = {{140, 20}, {35, 20} };
   NSTextField *fontview = [[[NSTextField alloc] initWithFrame:rectfont] autorelease];
   [view addSubview:fontview];
   [fontview setIntValue:printout_fontsize];
 //color  
-  NSRect pdfrect = {190, -10, 80, 70 };
+  NSRect pdfrect = {{190, -10}, {80, 70} };
   NSMatrix *pdfmatrix = [[[NSMatrix alloc] initWithFrame:pdfrect mode:NSRadioModeMatrix 
 						cellClass:[NSButtonCell class]
 					 numberOfRows:3 numberOfColumns:1] autorelease];
   [pdfmatrix setAllowsEmptySelection:NO];
+  NSSize spacing= {0, 2};
+  [pdfmatrix setIntercellSpacing:spacing];
+  NSSize cellsize= {80, 16};
+  [pdfmatrix setCellSize:cellsize];
   NSButton *color = [[[NSButton alloc] init] autorelease];
   [color setButtonType:NSRadioButton];
   [color setTitle:@"PDF color"];
-  [color sizeToFit];
   [pdfmatrix putCell:[color cell] atRow:0 column:0];
   NSButton *bw = [[[NSButton alloc] init] autorelease];
   [bw setButtonType:NSRadioButton];
   [bw setTitle:@"PDF B&W"];
-  [bw sizeToFit];
   [pdfmatrix putCell:[bw cell] atRow:1 column:0];
   NSButton *textfile = [[[NSButton alloc] init] autorelease];
   [textfile setButtonType:NSRadioButton];
   [textfile setTitle:@"Text File"];
-  [textfile sizeToFit];
   [pdfmatrix putCell:[textfile cell] atRow:2 column:0];
   [pdfmatrix selectCell:[bw cell]];
   if([pdfmatrix selectedRow] != printout_black) [pdfmatrix selectCellAtRow:printout_black column:0];
   [view addSubview:pdfmatrix];
 //paper format  
-  NSRect paperrect = {280, -10, 80, 70 };
+  NSRect paperrect = {{280, -10}, {80, 70} };
   NSMatrix *papermatrix = [[[NSMatrix alloc] initWithFrame:paperrect mode:NSRadioModeMatrix 
 						 cellClass:[NSButtonCell class]
 					      numberOfRows:2 numberOfColumns:1] autorelease];
   [papermatrix setAllowsEmptySelection:NO];
+  [papermatrix setIntercellSpacing:spacing];
+  [papermatrix setCellSize:cellsize];
   NSButton *A4 = [[[NSButton alloc] init] autorelease];
   [A4 setButtonType:NSRadioButton];
   [A4 setTitle:@"A4"];
-  [A4 sizeToFit];
   [papermatrix putCell:[A4 cell] atRow:0 column:0];
   NSButton *LETTER = [[[NSButton alloc] init] autorelease];
   [LETTER setButtonType:NSRadioButton];
   [LETTER setTitle:@"LETTER"];
-  [LETTER sizeToFit];
   [papermatrix putCell:[LETTER cell] atRow:1 column:0];
   [papermatrix selectCell:[LETTER cell]];
   int desired_paper_cell = (printout_pageformat == Fl_Paged_Device::A4 ? 0 : 1);
   if ([papermatrix selectedRow] != desired_paper_cell) [papermatrix selectCellAtRow:desired_paper_cell column:0];
   [view addSubview:papermatrix];
-  // landscape button
-  NSRect landscaperect = {280, 10, 80, 20 };
+// landscape button
+  NSRect landscaperect = {{280, 6}, {80, 20} };
   NSButton *landscape = [[[NSButton alloc] initWithFrame:landscaperect] autorelease];
   [landscape setButtonType:NSSwitchButton];
   [landscape setTitle:@"landscape"];
@@ -172,8 +165,8 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
   [landscape setIntValue:(printout_layout == Fl_Paged_Device::LANDSCAPE)];
   [landscape setEnabled:YES];
   [view addSubview:landscape];
-  //variable site button  
-  NSRect variablerect = {360, 30, 80, 20 };
+//variable site button  
+  NSRect variablerect = {{360, 30}, {80, 20} };
   NSButton *variable = [[[NSButton alloc] initWithFrame:variablerect] autorelease];
   [variable setButtonType:NSSwitchButton];
   [variable setTitle:@"variable sites only"];
@@ -187,8 +180,8 @@ char *mac_GetOutputFName_Plus(const char *dfault, const char *message, int use_o
   NSWindow *key = [NSApp keyWindow];
   int retval;
   if (fl_mac_os_version >= 100600) {
-    if (dir) [_panel setDirectoryURL:[NSURL fileURLWithPath:dir]];
-    if (fname) [_panel setNameFieldStringValue:fname];
+    if (dir) [_panel performSelector:@selector(setDirectoryURL:) withObject:[NSURL fileURLWithPath:dir]];
+    if (fname) [_panel performSelector:@selector(setNameFieldStringValue:) withObject:fname];
     retval = [_panel runModal];
   }
   else {
@@ -255,8 +248,25 @@ void delete_windowmenuitem(int rank)
   fl_sys_menu_bar->remove(rank);
 }
 
+static void file_receive_cb(const char *fname)
+{
+  SEA_VIEW *view;
+  Fl_Window *w = Fl::first_window();
+#if 100*FL_MAJOR_VERSION + 10*FL_MINOR_VERSION  + FL_PATCH_VERSION <= 131
+  // without this show() call, the opening of the alignment window is delayed by several seconds.
+  // not needed with FLTK 1.3.2 or above
+  if (fl_mac_os_version >= 100800 && w) w->show();
+#endif
+  while(w != NULL) {
+    const char *c = w->xclass();
+    if(c != NULL && strcmp(c, SEAVIEW_WINDOW) == 0) break;
+    w = Fl::next_window(w);
+  }
+  view = (w != NULL ? (SEA_VIEW *)w->user_data() : NULL);
+  use_initial_file(view, (char *)fname, false);
+}
 
-void show_apropos(Fl_Widget *w, void *unused)
+static void show_apropos(Fl_Widget *w, void *unused)
 {
   static Fl_Window *about = NULL;
   char line[100], *p;
@@ -294,11 +304,13 @@ void show_apropos(Fl_Widget *w, void *unused)
 
 void MG_apple_inits(void)
 {
+  fl_open_callback(file_receive_cb);
+  fl_mac_set_about(show_apropos, NULL, 0);
   Fl::set_font(FL_COURIER,"Courier");
-  Fl::set_font(FL_COURIER_BOLD,"Courier Bold");
+  Fl::set_font(FL_COURIER_BOLD, (fl_mac_os_version >= 100500 ? "Courier-Bold" : "Courier Bold") );
   if (fl_mac_os_version >= 100500) { // it seems that Courier Oblique is not present on older OS
-    Fl::set_font(FL_COURIER_ITALIC,"Courier Oblique");
-    Fl::set_font(FL_COURIER_BOLD_ITALIC,"Courier Bold Oblique");
+    Fl::set_font(FL_COURIER_ITALIC,"Courier-Oblique");
+    Fl::set_font(FL_COURIER_BOLD_ITALIC,"Courier-BoldOblique");
     }
 }
 
@@ -526,7 +538,8 @@ void Fl_Copy_Surface::MyCopyPictToClipboard()
 				      /* kUTTypePICT */ CFSTR("com.apple.pict"),
 				      data,
 				      kPasteboardFlavorNoFlags );
-  CFRelease (data);    
+  CFRelease (data);
+  if (fl_mac_os_version < 100400) return; // useful under 10.3 for unclear reason
   
   myqdpict = MyPictToQDPict();
   
@@ -542,7 +555,7 @@ void Fl_Copy_Surface::MyCopyPictToClipboard()
     (void) PasteboardPutItemFlavor (clipboard, (PasteboardItemID)1, 
 				     /* kUTTypePDF */ CFSTR("com.adobe.pdf"), 
 				     pdfdata, kPasteboardFlavorNoFlags);
-    }
+  }
   CFRelease (pdfdata); 
   CFRelease (clipboard);
   if(myqdpict != NULL) QDPictRelease(myqdpict);
@@ -565,12 +578,13 @@ void Fl_Copy_Surface::init_PDF_context(int w, int h)
   CGDataConsumerRelease (myconsumer);
 }
 
-//next 2 functions copy quartz data to clipboard both as pdf and as PICT (or tiff for 64-bit) bitmap
+//next 2 functions copy quartz data to clipboard both as pdf and as tiff bitmap
 void Fl_Copy_Surface::prepare_copy_pdf_and_pict(int w, int h)
 {
   init_PDF_context(w, h);
   if (gc == NULL) return;
-  CGContextBeginPage (gc, NULL);
+  CGRect bounds = CGRectMake(0, 0, w, h );	
+  CGContextBeginPage (gc, &bounds);
   CGContextTranslateCTM(gc, 0, h);
   CGContextScaleCTM(gc, 1.0f, -1.0f);
 }
@@ -586,89 +600,55 @@ void Fl_Copy_Surface::complete_copy_pdf_and_pict()
   PasteboardPutItemFlavor (clipboard, (PasteboardItemID)1, 
 						   CFSTR("com.adobe.pdf"), // kUTTypePDF
 						   pdfdata, kPasteboardFlavorNoFlags);
-  //second, transform this PDF to a bitmap image and put it as PICT/tiff in clipboard
+  //second, transform this PDF to a bitmap image and put it as tiff in clipboard
   CGDataProviderRef prov;
-  if(fl_mac_os_version >= 100400) { // true from 10.4
-    prov = CGDataProviderCreateWithCFData(pdfdata);
-    }
-  else {
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1040
+  if(fl_mac_os_version >= 100400)
+    prov = CGDataProviderCreateWithCFData(pdfdata); // 10.4
+  else 
+#endif
     prov = CGDataProviderCreateWithData(NULL, CFDataGetBytePtr(pdfdata), CFDataGetLength(pdfdata), NULL);
-  }
   CGPDFDocumentRef pdfdoc = CGPDFDocumentCreateWithProvider(prov);
   CGPDFPageRef pdfpage = CGPDFDocumentGetPage(pdfdoc, 1);
   CGRect clip = CGPDFPageGetBoxRect(pdfpage, kCGPDFCropBox);
   w = (int)(clip.size.width + 0.5);
   h = (int)(clip.size.height + 0.5);
   CGDataProviderRelease(prov);
-  CGColorSpaceRef space = CGColorSpaceCreateWithName(
-				fl_mac_os_version >= 100400 ? kCGColorSpaceGenericRGB : kCGColorSpaceUserRGB	     
-						     );
+  CGColorSpaceRef space = CGColorSpaceCreateDeviceRGB();
   const int scale = 2;
   w *= scale; h *= scale;
-  void *mem = malloc(w * h * sizeof(int));
-  gc = CGBitmapContextCreate(mem, w, h, 8, w * 4, space, kCGImageAlphaPremultipliedFirst /*| kCGBitmapByteOrder32Host*/);
+  void *mem = ( fl_mac_os_version >= 100600 ? NULL : malloc(w * h * 4) );
+  gc = CGBitmapContextCreate(mem, w, h, 8, w * 4, space, kCGImageAlphaNoneSkipLast );
   CFRelease(space);
-  if (gc == NULL) { free(mem); return; }
+  if (gc == NULL) { if (mem) free(mem); CGPDFDocumentRelease(pdfdoc); return; }
   CGRect rect = CGRectMake(0, 0, w, h);
   CGContextSetRGBFillColor(gc,  1,1,1,1);//need to clear background
   CGContextFillRect(gc, rect);
+  CGContextConcatCTM(gc, CGAffineTransformMakeScale(scale, scale));
 #if MAC_OS_X_VERSION_MAX_ALLOWED >= 1030
-  if (CGContextDrawPDFPage) {
-    CGContextConcatCTM(gc, CGAffineTransformMakeScale(scale, scale));
-    CGContextDrawPDFPage(gc, pdfpage);  // 10.3
-    }
-  else
+  CGContextDrawPDFPage(gc, pdfpage);  // 10.3
 #endif
-    CGContextDrawPDFDocument(gc, rect, pdfdoc, 1);
   CGPDFDocumentRelease(pdfdoc);
   CFRelease(pdfdata);
-  if(fl_mac_os_version >= 100400) {// way for >= 10.4 that doesn't use QuickTime:
-    CFMutableDataRef pictdata = CFDataCreateMutable(NULL, 0);
-#ifdef USE_PICT
-    CFStringRef  uti = CFSTR("com.apple.pict");
-#else
-    CFStringRef  uti = CFSTR("public.tiff");
+  unsigned char *plane = (unsigned char *)CGBitmapContextGetData(gc);
+  NSBitmapImageRep *rep = [[NSBitmapImageRep alloc] initWithBitmapDataPlanes:&plane
+								  pixelsWide:w
+								  pixelsHigh:h
+							       bitsPerSample:8
+							     samplesPerPixel:3
+								    hasAlpha:NO
+								    isPlanar:NO
+							      colorSpaceName:NSDeviceRGBColorSpace
+								 bytesPerRow:w*4
+								bitsPerPixel:32];
+  CFDataRef data = (CFDataRef)[rep TIFFRepresentation];
+  [rep release];
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 1030
+  PasteboardPutItemFlavor(clipboard, (PasteboardItemID)1, CFSTR("public.tiff"), data, kPasteboardFlavorNoFlags );
 #endif
-    CGImageRef image = CGBitmapContextCreateImage(gc);
-    CGImageDestinationRef dest = CGImageDestinationCreateWithData(pictdata, uti, 1, NULL);
-    CGImageDestinationAddImage(dest, image, NULL);
-    CGImageDestinationFinalize(dest);
-    CGImageRelease(image);
-    CFRelease(dest);
-#ifdef USE_PICT
-    CFRange range = CFRangeMake(0, 512);
-    CFDataDeleteBytes(pictdata, range);// The PICT clipboard REQUIRES only the image data, not the header.
-#endif
-    PasteboardPutItemFlavor(clipboard, (PasteboardItemID)1, uti, pictdata, kPasteboardFlavorNoFlags );
-    CFRelease (pictdata);    
-    }
-  else {// way for < 10.4 that uses QuickTime:
-#ifdef __ppc__
-    GraphicsExportComponent exporter;
-    OSErr err = OpenADefaultComponent(GraphicsExporterComponentType, kQTFileTypePicture /*'PICT'*/, &exporter);
-    err = GraphicsExportSetInputCGBitmapContext(exporter, gc);
-    Handle dataHandle = NewHandle(0);
-    err = GraphicsExportSetOutputHandle(exporter, dataHandle);
-    unsigned long size;
-    err = GraphicsExportDoExport(exporter, &size);
-    err = CloseComponent(exporter); 
-    if(GetHandleSize(dataHandle) > 512) {
-      HLock(dataHandle);
-      // The clipboard REQUIRES only the image data, not the header.
-      CFDataRef pictdata = CFDataCreate(NULL, (const UInt8 *)*dataHandle + 512, GetHandleSize(dataHandle) - 512);
-      HUnlock(dataHandle);
-      PasteboardPutItemFlavor(clipboard, (PasteboardItemID)1, 
-						      CFSTR("com.apple.pict"),
-						      pictdata,
-						      kPasteboardFlavorNoFlags ); 
-      CFRelease (pictdata);    
-    }
-    DisposeHandle(dataHandle);
-#endif
-  }
   CFRelease (clipboard);    
   CGContextRelease(gc);
-  free(mem);
+  if (mem) free(mem);
 }
 
 void mac_tmp_dir_name(char *buffer, int l)
