@@ -249,7 +249,7 @@ while (!is_block_end(p));
 int process_taxa_dim(FILE *in)
 {
 char *p;
-int ntax;
+int ntax = 0;
 
 p = next_token(in);
 if(p == NULL) return 0;
@@ -297,7 +297,7 @@ int process_chars_format(FILE *in, int *protein, int *interleave,
 	int *matchchar, int *missing)
 {
 char *p;
-int gap;
+int gap=0;
 *interleave = FALSE;
 
 do	{
@@ -626,7 +626,7 @@ char **process_block_data(FILE *in, int *pntaxa, char ***ptaxnames,
 		int *nsites, int *protein)
 {
 char **seqs, **taxnames, *p;
-int gap, interleave = FALSE, matchchar = 0, missing = 0;
+int gap=0, interleave = FALSE, matchchar = 0, missing = 0;
 
 seqs = NULL; *nsites = 0; taxnames = NULL; *pntaxa = 0;
 while(TRUE) {
@@ -664,7 +664,7 @@ char **process_block_characters(FILE *in, int *pntaxa, char ***taxnames,
 int *nsites, int *protein)
 {
 char *p, **seqs;
-int interleave, gap, matchchar = 0, missing = 0;
+int interleave=0, gap, matchchar = 0, missing = 0;
 
 seqs = NULL; *nsites = 0;
 while(TRUE) {
@@ -784,7 +784,7 @@ char *replace_ranks_by_names(char *tree, int ntaxa, char **taxnames)
       r = bootstrap_comment;
       do {
 	*r = *p++;
-	if (r - bootstrap_comment < sizeof(bootstrap_comment) - 1) r++;
+	if ((unsigned)(r - bootstrap_comment) < sizeof(bootstrap_comment) - 1) r++;
       }
       while (*p != ']');
       p++;
@@ -1089,7 +1089,7 @@ return number;
 int process_unal_format(FILE *in)
 {
 char *p;
-int protein;
+int protein = 0;
 
 p = next_token(in);
 if(p!=NULL)majuscules(p);
