@@ -40,7 +40,6 @@ static int scan_list(raa_db_access *raa, int mylist, const char *scan_word, cons
 
 
 extern void interrupt_callback(Fl_Widget *ob, void *data);
-extern int add_seq_to_align(SEA_VIEW *view, char *newname, char *newseq, int lenseq);
 extern int compute_size_params(SEA_VIEW *view, int force_recompute);
 extern void update_comment_gc(int seqnum, int ncbigc, SEA_VIEW *view);
 extern char *get_res_value(const char *name, const char *def_value);
@@ -484,6 +483,7 @@ raa_db_access *open_acnuc_socket(Fl_Widget *ok, Fl_Button *close_b)
     close_b->label("Close");
     if (ss->killed) {
       fl_alert("%s", "Connection attempt was cancelled");
+      delete ss;
       return NULL;
       }
     err = ss->err;
