@@ -50,12 +50,8 @@ extern char *my_fgets(char *s, int n, FILE *f);
 #endif
 
 #ifdef WIN32
-#include <wchar.h>
-typedef const wchar_t *LPCWSTR;
-extern char *wchartoutf8(LPCWSTR in);
-extern LPCWSTR utf8towchar(const char *in);
 extern void my_bell(void); 
-#define fopen(a,b) _wfopen(utf8towchar(a), L##b)
+#define fopen(a,b) fl_fopen(a, b)
 #define memccpy _memccpy
 #elif defined __APPLE__
 #else
@@ -327,8 +323,7 @@ extern void draw_region_background(SEA_VIEW *view, int f_seq0, int l_seq0);
 extern void load_seq_dialog(SEA_VIEW *view);
 extern void edit_comments_dialog(SEA_VIEW *view);
 extern void edit_sequence_dialog(SEA_VIEW *view);
-extern int add_seq_to_align(SEA_VIEW *view, char *newname, char *newseq, 
-	int lenseq);
+extern int add_seq_to_align(SEA_VIEW *view, char *newname, char *newseq, int lenseq, bool no_gui = false);
 extern int get_color_for_base( int key );
 extern int get_color_for_aa( int key );
 extern char **prepcolranks(char **seq, int totseqs, int maxlen, int *eachlength,
@@ -336,7 +331,7 @@ extern char **prepcolranks(char **seq, int totseqs, int maxlen, int *eachlength,
 extern char complement_base(char old);
 extern void select_deselect_seq(SEA_VIEW *view, int new_seq, int from_tree = 0);
 extern void save_active_region(SEA_VIEW *view);
-extern void align_selected_parts(SEA_VIEW *view, int align_algorithm, align_choice my_align_choice);
+extern void align_selected_parts(SEA_VIEW *view, align_choice my_align_choice, bool no_gui = false);
 extern void free_region(region *reg);
 extern void my_watch_cursor(Fl_Window *win);
 extern void update_current_seq_length(int newlength, SEA_VIEW *view);
