@@ -24,7 +24,7 @@ void gaps_as_unknown_states(allseq *phyml_seqs, int protein);
 void free_after_view_to_allseq(allseq *phyml_seqs);
 void initialize_random_series(void);
 void bootstrap_weights(allseq *seqs, int use_codons);
-void savedistinfile(matrix *mat, const char *dirname);
+void savedistinfile(matrix *mat, const char *dirname, const char *outfname);
 matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap, char **p_err_mess);
 void phyml_dialog(SEA_VIEW *view);
 static void change_searching_callback(Fl_Widget *ob, void *data);
@@ -32,10 +32,10 @@ static void cancel_callback(Fl_Widget *ob, void *data);
 static void model_callback(Fl_Widget *ob, void *data);
 void run_phyml_callback(Fl_Widget *ob, void *data);
 void distance_method_dialog(SEA_VIEW *view);
-matrix *run_calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap, char **p_err_mess);
-void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int use_bionj, int use_bootstrap,
-						 int replicates, Fl_Box *box, const char *distance_name, int using_kaks,
-						 int savetofile, const char *user_tree);
+matrix *run_calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap, char **p_err_mess, int no_gui);
+char *run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int use_bionj, int use_bootstrap,
+			 int replicates, Fl_Box *box, const char *distance_name, int using_kaks,
+			 const char* distancefname, const char *user_tree, int no_gui, void (*alert)(const char*, ...) );
 char *bootstrap_reformat(char *tree, int replicates);
 matrix *Obs_Dist(allseq *data, model *mod);
 matrix *Kimura_p_Dist(allseq *data);
@@ -46,11 +46,14 @@ double hky(char* seq1, char* seq2, int lg, int *wght);
 int freq_obs(char* seq1, char* seq2, double* freq, int lgseq, int *wght);
 int informative(char** sequ, int nb);
 void jumble_fct(char** seq, char** name, int notu);
+char *run_parsimony_method(SEA_VIEW *view, int nogaps, const char *gapsunknown, int nreplicates, int njumbles, 
+			   int maxbest, int user_tree, Fl_Box *w_count,
+			   const char **pprogname, int no_gui, void (*alert)(const char *,...) );
 void parsimony_dialog(SEA_VIEW *view);
 char *parsimony_with_jumble(char **pars_seqs, char **pars_names, int n_otu, int njumbles, int protein, int *psteps,
-	int *pcount, int maxbest, Fl_Widget *w_count);
+	int *pcount, int maxbest, Fl_Widget *w_count, int no_gui);
 char *parsimony_with_bootstrap(allseq *tmpallseq, char **pars_seqs, char **pars_names, int n_otu, int njumbles, 
-	int protein, int *psteps, int maxbest, int replicates, Fl_Box *w_count, int *preplicates_done);
+	int protein, int *psteps, int maxbest, int replicates, Fl_Box *w_count, int *preplicates_done, int no_gui);
 double *vector(long nl, long nh);
 void free_vector(double *v, long nl, long nh);
 int ludcmp(double **a, int n, int *indx, double *d);
@@ -59,6 +62,7 @@ double logdet(char* seq1, char* seq2, int lg, int *wght);
 matrix *LOGDET_dist(allseq *data);
 char *put_names_back_in_tree(char *oldtree, char **names);
 char *replace_tree_names_by_rank(const char *tree, char **names, int notu);
+void command_line_phylogeny(int argc, char **argv);
 
 
 extern void interrupt_callback(Fl_Widget *ob, void *data);
@@ -79,8 +83,13 @@ extern int set_res_value(const char *name, const char *value);
 extern char *create_tmp_filename(void);
 extern void delete_tmp_filename(const char *base_fname);
 extern int is_view_valid(SEA_VIEW *view);
-char *parsimony(char** seq, char** seqname, int notu, int* steps, char* toevaluate, int arg_maxtrees, int *bt_weights, bool);
+char *parsimony(char** seq, char** seqname, int notu, int* steps, char* toevaluate, int arg_maxtrees, 
+		int *bt_weights, bool protein, int no_gui);
 extern int fl_create_thread(void (*f) (void *), void* p, unsigned);
+extern char *argname(int argc, char *argv[], const char *arg);
+extern float argval(int argc, char *argv[], const char *arg, float defval);
+extern bool isarg(int argc, char *argv[], const char *arg);
+extern SEA_VIEW *cmdline_read_input_alignment(int argc, char **argv);
 extern "C" {
   void prefastlwl(void);
   void loadrl(void);
@@ -97,8 +106,7 @@ void trees_callback(Fl_Widget *ob, void *data)
  */
 {
 	SEA_VIEW *view = (SEA_VIEW *)ob->user_data();
-	//int reponse = (int)data;
-  int reponse = view->menubar->mvalue() - view->menu_trees->get_menu();
+	int reponse = ((Fl_Menu_*)ob)->mvalue() - view->menu_trees->get_menu();
 	if(reponse <= 2) {
 		if(view->tot_seqs < 3 || (view->tot_sel_seqs > 0 && view->tot_sel_seqs < 3) ) {
 			fl_alert("Can't draw tree for less than 3 sequences");
@@ -115,7 +123,17 @@ void trees_callback(Fl_Widget *ob, void *data)
 			}
 		}
 	if(reponse == 0) {//Parsimony methods 
-		parsimony_dialog(view);
+	  for (int i = 0; i < view->tot_seqs; i++) {//check that no repeated sequence names
+	    if (view->tot_sel_seqs > 0 && !view->sel_seqs[i]) continue;
+	    for (int j = i+1; j < view->tot_seqs; j++) {
+	      if (view->tot_sel_seqs > 0 && !view->sel_seqs[j]) continue;
+	      if (strcmp(view->seqname[i], view->seqname[j]) == 0) {
+		fl_alert("The parsimony tree can't be computed if different sequences (#%d, #%d) have the same name: %s", i+1, j+1, view->seqname[i]);
+		return;
+		}
+	      }
+	    }
+	    parsimony_dialog(view);
 	}
 	else if(reponse == 1) {//Distance methods 
 		distance_method_dialog(view);
@@ -146,7 +164,7 @@ void trees_callback(Fl_Widget *ob, void *data)
 		treedraw(NULL, view, NULL, FALSE);
 		}
 	else {
-	  reponse = view->menubar->mvalue() - view->menu_trees->vitem(0);
+	        reponse = ((Fl_Menu_*)ob)->mvalue() - view->menu_trees->vitem(0);
 		char *p = view->trees[reponse];
 		Fl_Window *w = Fl::first_window();
 		while(w != NULL) {
@@ -276,17 +294,31 @@ void distance_method_dialog(SEA_VIEW *view)
 	distance_choice = distance->value();
 	int using_kaks = ((!view->protein) && (distance_choice == Ka || distance_choice == Ks) );
 	if(!tree_build_interrupted) {
-		sscanf(replicates->value(), "%d", &def_replicates);
-		interrupt->label("Interrupt");
-		if(bootstrap->value()) {
-			box->label("Count: 0");
-			}
-		go->hide();
-		Fl::flush();
-		run_distance_method(view, distance_choice, ignore_all_gaps->value(), bionj->value(), 
-							bootstrap->value(), def_replicates, box, distance->text(), using_kaks,
-							savetofile->value(), usertree->value() ? view->trees[treechoice->value()] : NULL);
-		}
+	  sscanf(replicates->value(), "%d", &def_replicates);
+	  interrupt->label("Interrupt");
+	  if(bootstrap->value()) {
+		  box->label("Count: 0");
+		  }
+	  go->hide();
+	  Fl::flush();
+	  char *tree = run_distance_method(view, distance_choice, ignore_all_gaps->value(), bionj->value(), 
+						  bootstrap->value(), def_replicates, box, distance->text(), using_kaks,
+			      savetofile->value()?"":NULL, usertree->value() ? view->trees[treechoice->value()] : NULL,
+			      false, fl_alert);
+	  const char *choice = bionj->value()? "BioNJ_tree" : "NJ_tree";
+	  const char *n = extract_filename(view->masename);
+	  char *title = NULL;
+	  if (usertree->value()) { title = new char[20]; strcpy(title, "Least_squares_brl");}
+	  else if (n) {
+	    char *q = (char*)strchr(n, '.');
+	    if (q) *q = 0;
+	    title = new char[strlen(n) + 12];
+	    sprintf(title, "%s-%s", n, choice);
+	    if (q) *q = '.';
+	  }
+	  if (tree) treedraw(tree, view, n || usertree->value() ? title:choice, FALSE);
+	  delete[] title;
+	}
 	w->hide();
 }
 
@@ -314,7 +346,7 @@ void prep_calc_dist_matrix(dist_struct *ds)
   Fl::unlock();
 }
 
-matrix *run_calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap, char **p_err_mess)
+matrix *run_calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap, char **p_err_mess, int no_gui)
 {
   dist_struct *ds = new dist_struct;
   ds->seqs = seqs;
@@ -326,8 +358,13 @@ matrix *run_calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwl
   ds->result = NULL;
   ds->done = false;
   tree_build_interrupted = false;
-  fl_create_thread( (void (*)(void*))prep_calc_dist_matrix, ds, 0);
-  while (!ds->done) Fl::wait();
+  if (no_gui) {
+    ds->result = calc_dist_matrix(ds->seqs, ds->distkind, ds->protein, ds->lwlseqs, ds->in_bootstrap, &(ds->err_mess));
+  }
+  else {
+    fl_create_thread( (void (*)(void*))prep_calc_dist_matrix, ds, 0);
+    while (!ds->done) Fl::wait();
+  }
   matrix* m = ds->result;
   if (p_err_mess) *p_err_mess = ds->err_mess;
   delete ds;
@@ -335,23 +372,23 @@ matrix *run_calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwl
 }
 
 
-void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int use_bionj, int use_bootstrap,
-	int replicates, Fl_Box *box, const char *distance_name, int using_kaks, int savetofile,
-	const char *user_tree)
+char *run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int use_bionj, int use_bootstrap,
+	int replicates, Fl_Box *box, const char *distance_name, int using_kaks, const char* distancefname,
+	const char *user_tree, int no_gui, void (*alert)(const char*, ...) )
 {
 	char *display_tree, *err_mess, tree_label[100]="";
 	int total = 0, i, j;
 	char **lwlseqs = NULL;
-	if(using_kaks) remove_all_gaps = FALSE;//keep gaps not to alter reading frame
+  	if(using_kaks) remove_all_gaps = FALSE;//keep gaps not to alter reading frame
 	allseq *phyml_seqs = view_to_allseq(view, remove_all_gaps);
 	if(use_bootstrap) {//for bootstrap, seq names must be all distinct
 		for(i = 0; i < phyml_seqs->n_otu - 1; i++) {
 			for(j = i+1; j < phyml_seqs->n_otu; j++) {
 				if(strcmp(phyml_seqs->c_seq[i]->name, phyml_seqs->c_seq[j]->name) == 0) {
-					fl_alert("Can't run bootstrap because sequence name %s is used twice", 
+					alert("Can't run bootstrap because sequence name %s is used twice", 
 							 phyml_seqs->c_seq[i]->name);
 					free_after_view_to_allseq(phyml_seqs);
-					return;
+					return NULL;
 					}
 				}
 			}
@@ -360,18 +397,18 @@ void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 		lwlseqs = (char **)malloc(phyml_seqs->n_otu * sizeof(char *));
 		for(i = 0; i < phyml_seqs->n_otu; i++) lwlseqs[i] = phyml_seqs->c_seq[i]->state;
 		}
-	matrix *phyml_mat = run_calc_dist_matrix(phyml_seqs, distkind, view->protein, lwlseqs, FALSE, &err_mess);
+	matrix *phyml_mat = run_calc_dist_matrix(phyml_seqs, distkind, view->protein, lwlseqs, FALSE, &err_mess, no_gui);
 	if(phyml_mat == NULL) {
 	  free_after_view_to_allseq(phyml_seqs);
 	  if( using_kaks ) {
 	    free(lwlseqs);
 	    if (err_mess) {
-	      fl_alert("%s", err_mess);
+	      alert("%s", err_mess);
 	      delete[] err_mess;
 	      }
 	    }
-	  else if (!tree_build_interrupted) fl_alert("Can't compute distances because sequences are too divergent.\nAre they aligned?");
-	  return;
+	  else if (!tree_build_interrupted) alert("Can't compute distances because sequences are too divergent.\nAre they aligned?");
+	  return NULL;
 	  }
 	if(user_tree != NULL) {//process user tree
 		extern char *least_squares_brl(const char *tree, int nbseq, double **dist, char **names);
@@ -390,22 +427,21 @@ void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 		p = least_squares_brl(p, phyml_seqs->n_otu, phyml_mat->dist, usednames);
 		if(view->tot_sel_seqs > 0) free(usednames);
 		if(p != NULL) {
-			char *fulltree = (char *)malloc(strlen(p) + strlen(distance_name) + 60);
-			sprintf(fulltree, "[User-tree w/ least-squares br lengths, %d sites %s]%s", 
+			display_tree = (char *)malloc(strlen(p) + strlen(distance_name) + 60);
+			sprintf(display_tree, "[User-tree w/ least-squares br lengths, %d sites %s]%s", 
 					phyml_seqs->clean_len, distance_name, p);
 			free(p);
-			box->window()->hide();//necessary under X11 so tree is in foreground
-			treedraw(fulltree, view, "Least_squares_brl", FALSE);
+			if (box) box->window()->hide();//necessary under X11 so tree is in foreground
 			}
 		Free_Mat(phyml_mat);
 		free_after_view_to_allseq(phyml_seqs);
-		return;
+		return display_tree;
 		}
-	if(savetofile) {
-		savedistinfile(phyml_mat, extract_dirname(view->masename));
+	if(distancefname) {
+		savedistinfile(phyml_mat, extract_dirname(view->masename), no_gui ? distancefname : NULL);
 		Free_Mat(phyml_mat);
 		free_after_view_to_allseq(phyml_seqs);
-		return;
+		return NULL;
 		}
 	phyml_mat->method = !use_bionj; //1:NJ 0:BioNJ
 	phyml_mat->tree = Make_Tree_From_Scratch(phyml_seqs->n_otu, phyml_seqs);
@@ -422,7 +458,7 @@ void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 			for(int r = 0; r < replicates; r++) {
 				if (tree_build_interrupted) break;
 				bootstrap_weights(phyml_seqs, using_kaks );
-				phyml_mat = run_calc_dist_matrix(phyml_seqs, distkind, view->protein, lwlseqs, TRUE, &err_mess);
+				phyml_mat = run_calc_dist_matrix(phyml_seqs, distkind, view->protein, lwlseqs, TRUE, &err_mess, no_gui);
 				if(phyml_mat == NULL) {
 					if (!tree_build_interrupted) total = 0;
 					break;
@@ -437,9 +473,11 @@ void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 				free(one_replicate_tree);
 				char newbox[20];
 				sprintf(newbox,"Count: %d", total);
-				box->label(newbox);
-				box->redraw();
-				Fl::wait(0);
+				if (box) {
+				  box->label(newbox);
+				  box->redraw();
+				  Fl::wait(0);
+				}
 			}
 		}
 		if(total > 0) {
@@ -458,40 +496,31 @@ void run_distance_method(SEA_VIEW *view, int distkind, int remove_all_gaps, int
 		free(lwlseqs);
 		}
 	if(display_tree == NULL) {
-	  if( !using_kaks ) fl_alert("Can't compute distances because sequences are too divergent.\nAre they aligned?");
-	  return;
+	  if( !using_kaks) alert("Can't compute distances because sequences are too divergent.\nAre they aligned?");
+	  return NULL;
 	}
 	//add label to tree
 	char *tree = (char *)malloc(strlen(tree_label) + strlen(display_tree) + 4);
 	sprintf(tree, "[%s] %s", tree_label, display_tree);
 	free(display_tree);
-	box->window()->hide();//necessary under X11 so tree is in foreground
-  const char *choice = use_bionj? "BioNJ_tree" : "NJ_tree";
-  const char *n = extract_filename(view->masename);
-  char *title = NULL;
-  if (n) {
-    char *q = (char*)strchr(n, '.');
-    if (q) *q = 0;
-    title = new char[strlen(n) + 12];
-    sprintf(title, "%s-%s", n, choice);
-    if (q) *q = '.';
-    }
-  treedraw(tree, view, n ? title:choice, FALSE);
-  delete[] title;
+	if (box) box->window()->hide();//necessary under X11 so tree is in foreground
+  return tree;
 }
 
-void savedistinfile(matrix *mat, const char *dirname)
+void savedistinfile(matrix *mat, const char *dirname, const char *outfname)
 {
 	FILE *out;
 	int i, j;
-	Fl_Native_File_Chooser *chooser = new Fl_Native_File_Chooser();
-	chooser->type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);
-  chooser->options(Fl_Native_File_Chooser::SAVEAS_CONFIRM | chooser->options());
-	chooser->title("Set distance output filename");  
-  chooser->directory(dirname);
-  chooser->preset_file("seqs.dist");
-	char *outfname = run_and_close_native_file_chooser(chooser);
-	if(outfname == NULL) return;
+	if (!outfname) {
+	  Fl_Native_File_Chooser *chooser = new Fl_Native_File_Chooser();
+	  chooser->type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);
+	  chooser->options(Fl_Native_File_Chooser::SAVEAS_CONFIRM | chooser->options());
+	  chooser->title("Set distance output filename");  
+	  chooser->directory(dirname);
+	  chooser->preset_file("seqs.dist");
+	  outfname = run_and_close_native_file_chooser(chooser);
+	  if(outfname == NULL) return;
+	}
 	out = fopen(outfname, "w");
 	if(out == NULL) return;
 	fputs("#distances order: d(1,2),...,d(1,n) <new line> d(2,3),...,d(2,n) <new line>...\n", out);
@@ -505,13 +534,22 @@ void savedistinfile(matrix *mat, const char *dirname)
 	for( i = 0; i < mat->n_otu ; i++) {
 		fprintf(out, "%s ", mat->name[i]);
 		}
-	fputs("\n", out);
-	fclose(out);
+	fputs("\n\n", out);
+  fputs("#pairwise distances\n", out);
+  for (i = 0; i < mat->n_otu; i++) {
+    for (j = i + 1; j < mat->n_otu; j++) {
+      char *n1 = mat->name[i];
+      char *n2 = mat->name[j];
+      if (strcmp(n1, n2) > 0) { char *p = n1; n1 = n2; n2 = p; }
+      fprintf(out, "%s,%s: %f\n", n1, n2, mat->dist[i][j]);
+    }
+  }
+  fclose(out);
 }
 
 matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap, char **p_err_mess)
 {
-	matrix *phyml_mat;
+	matrix *phyml_mat = NULL;
 	if(distkind == observed_pdist || distkind == observed_ndist) {//observed
 		model jcmodel;
 		jcmodel.stepsize = 1;
@@ -584,6 +622,22 @@ matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs
 	return phyml_mat;
 }
 
+/* a window where the return key always triggers a button
+ */
+class return_window : public Fl_Window {
+  Fl_Return_Button *run_button;
+public:
+  return_window(int w, int h, const char *t) : Fl_Window(w, h, t) {}
+  void run(Fl_Return_Button *b) { run_button = b; }
+  int handle(int event) {
+    if (event == FL_KEYBOARD && (Fl::event_key() == FL_Enter || Fl::event_key() == FL_KP_Enter)) {
+      run_button->do_callback();
+      return 1;
+      }
+    return Fl_Window::handle(event);
+    }
+};
+
 Fl_Input *name;
 Fl_Choice *mymodel;
 Fl_Round_Button *b_alrt; Fl_Round_Button *b_no; Fl_Int_Input *b_count;
@@ -605,27 +659,32 @@ void phyml_dialog(SEA_VIEW *view)
 		char *p, *q;
 		p = get_res_value("phyml",
 #ifdef WIN32
-					"PhyML_3.0_win32.exe"
+					"PhyML-3.1_win32.exe"
 #elif defined(__APPLE__)
 #ifdef __ppc__
 				  "phyml_3.0_universal"
 #else
-				  "PhyML_3.0_macOS_i386"
+				  "PhyML-3.1_macOS-MountainLion"
 #endif
 #elif defined(sparc)
 						  "phyml_3.0_sparc"
 #else
 #ifndef PHYMLNAME
 #ifdef __i386__
-#define PHYMLNAME "PhyML_3.0_linux32"
+#define PHYMLNAME "PhyML-3.1_linux32"
 #else
-#define PHYMLNAME "PhyML_3.0_linux64"
+#define PHYMLNAME "PhyML-3.1_linux64"
 #endif
 #endif
 					PHYMLNAME
 #endif
 		);
-		if((q = get_full_path(p)) == NULL) {
+#ifdef __APPLE__
+	  if (fl_mac_os_version < 100800) p = (char*)"phyml_3.0_universal";
+#elif defined(__linux__)
+	  if ((q = get_full_path(p)) == NULL) p = (char*)"phyml";
+#endif
+		if ((q = get_full_path(p)) == NULL) {
 			int rep = fl_choice("Seaview does not find PhyML under the name %s.\n"
 					  "Is PhyML available on your computer ?", "No", "Available", NULL, p);
 			if(rep == 0) return;
@@ -641,7 +700,7 @@ void phyml_dialog(SEA_VIEW *view)
 		phyml_path = (char *)malloc(strlen(q) + 3);
 		sprintf(phyml_path, "\"%s\"", q);
 		}
-	Fl_Window *w = new Fl_Window(270,130, "PhyML options");
+	return_window *w = new return_window(270,130, "PhyML options");
 	w->set_modal();
 	//model
 	mymodel = new Fl_Choice(50, y, 90, 20, "Model:");
@@ -815,6 +874,7 @@ void phyml_dialog(SEA_VIEW *view)
 	Fl_Button *cancel = new Fl_Button(3, y, 50, 20, "Cancel");
 	cancel->callback(cancel_callback, NULL);
 	Fl_Return_Button *ok = new Fl_Return_Button(w->w() - 60, y, 55, 20, "Run");
+	w->run(ok);
 	ok->callback(run_phyml_callback, view);
   u_quiet = new Fl_Check_Button(cancel->x() + cancel->w() + 40, y, 65, 20, "Quiet");
   u_quiet->value(0);
@@ -909,7 +969,7 @@ void run_phyml_callback(Fl_Widget *ob, void *data)
     tmpnames[i] = (char *)malloc(15);
     truenames[i] = strdup(view->seqname[i]);
     sprintf(tmpnames[i], "Seq%d_", i);
-    length = FL_max(length, strlen(tmpnames[i]));
+    length = FL_max(length, (int)strlen(tmpnames[i]));
     }
   status = save_phylip_file(input, tmpseqs,
 				   tmpnames, view->tot_seqs, view->each_length, pr, 
@@ -980,26 +1040,17 @@ void run_phyml_callback(Fl_Widget *ob, void *data)
 		   fputs(tree, out);
 		   free(tree);
 		   fclose(out);
-		   sprintf(args + strlen(args), " -u %s", tmp);
+		   sprintf(args + strlen(args), " -u \"%s\"", tmp);
 		  }
    }
   if(u_random->value()) {
 		  sprintf(args + strlen(args), " --rand_start --n_rand_starts %d", atoi(u_random_count->value()));
 		  }
   //finish with (long) input filename
-  sprintf(args + strlen(args), " -i %s", input);
+  sprintf(args + strlen(args), " -i \"%s\"", input);
   ob->window()->hide(); //deletion of this window should be done totally at the end of this callback
   status = run_external_prog_in_pseudoterm(args, NULL, "tree-building");
   free(args);
-  remove(input);
-  sprintf(input, "%s.startingtree", base_fname);
-  remove(input);
-  if(replicates > 0) {
-    sprintf(input, "%s.phy_phyml_boot_trees.txt", base_fname);
-    remove(input);
-    sprintf(input, "%s.phy_phyml_boot_stats.txt", base_fname);
-    remove(input);
-    }
   sprintf(input, "%s.phy_phyml_stats.txt", base_fname);
   in = fopen(input, "r");
   if (in == NULL) {
@@ -1016,7 +1067,6 @@ void run_phyml_callback(Fl_Widget *ob, void *data)
 		  else if( (p=strstr(line, "Log-likelihood :")) != NULL) sscanf(p+16, "%f", &logL);
 		  }
 	  fclose(in);
-	  remove(input);
 	  }
   sprintf(input, "%s.phy_phyml_tree.txt", base_fname);
   in = fopen(input, "r");
@@ -1064,7 +1114,6 @@ void run_phyml_callback(Fl_Widget *ob, void *data)
 	    delete[] title;
 	  }
 	}
-  remove(input);
   Fl::delete_widget(ob->window());
 way_out:
   for(i = 0; i < tot_seqs; i++) {
@@ -1377,7 +1426,6 @@ char *bootstrap_reformat(char *tree, int replicates)
 matrix *K80_dist_nogamma(allseq *data)
 {
 	int i,j,k;
-	int diff;
 	phydbl unc_len;
 	matrix *mat;
 	phydbl **len;
@@ -1404,7 +1452,6 @@ matrix *K80_dist_nogamma(allseq *data)
 				   ((data->c_seq[j]->state[i] == 'C' || data->c_seq[j]->state[i] == 'T') &&
 					(data->c_seq[k]->state[i] == 'A' || data->c_seq[k]->state[i] == 'G')))
 				{
-					diff++;
 					mat->Q[j][k]+=data->wght[i];
 					len[j][k]+=data->wght[i];
 					len[k][j]=len[j][k];
@@ -1416,7 +1463,6 @@ matrix *K80_dist_nogamma(allseq *data)
 					   ((data->c_seq[j]->state[i] == 'C' && data->c_seq[k]->state[i] == 'T') ||
 						(data->c_seq[j]->state[i] == 'T' && data->c_seq[k]->state[i] == 'C')))
 					{
-						diff++;
 						mat->P[j][k]+=data->wght[i];
 						len[j][k]+=data->wght[i];
 						len[k][j]=len[j][k];
@@ -1673,12 +1719,91 @@ void jumble_fct(char** seq, char** name, int notu)
 		}
 }
 
+char *run_parsimony_method(SEA_VIEW *view, int nogaps, const char *gapsunknown, int nreplicates, int njumbles, 
+			   int maxbest, int user_tree, Fl_Box *w_count,
+			   const char **pprogname, int no_gui, void (*alert)(const char *,...) )
+{
+  char *tree = 0, *final_tree = 0;
+  int i, steps, info, count, replicates_done;
+  const char *p;
+  allseq *tmpallseqs = view_to_allseq(view, nogaps);
+  if(*gapsunknown) gaps_as_unknown_states(tmpallseqs, view->protein);
+  char **pars_seqs = (char **)malloc(tmpallseqs->n_otu * sizeof(char *));
+  char **pars_names = (char **)malloc(tmpallseqs->n_otu * sizeof(char *));
+  for( i = 0; i < tmpallseqs->n_otu; i++) {
+    pars_seqs[i] = tmpallseqs->c_seq[i]->state;
+    pars_names[i] = tmpallseqs->c_seq[i]->name;
+  }
+  info = informative(pars_seqs, tmpallseqs->n_otu);
+  if(view->protein) *pprogname = "Protpars"; 
+  else {
+    *pprogname = "Dnapars";
+    if (tmpallseqs->clean_len >= SHRT_MAX) {
+      alert("Can't compute parsimony with DNA seqs longer than %d.", SHRT_MAX);
+      goto way_out;
+    }
+  }
+  if (nreplicates) {
+    tree = parsimony_with_bootstrap(tmpallseqs, pars_seqs, pars_names, tmpallseqs->n_otu, njumbles, 
+				    view->protein, &steps, maxbest, nreplicates, w_count, &replicates_done, no_gui);
+    if (tree) {
+      final_tree = (char *)malloc(strlen(tree) + 150);
+      sprintf(final_tree, "[%s, bootstrap with %d replic., %d steps, %d sites (%d informative)%s]%s",
+	      *pprogname, replicates_done, steps, tmpallseqs->clean_len, info, gapsunknown, tree);
+    }
+  }
+  else if(user_tree) {
+    char *differs, *input_tree;
+    p = view->trees[user_tree - 1];
+    while(*p == ' ') p++;
+    if(*p == '[') p = strchr(p, ']')+ 1;
+    while(*p == ' ') p++;
+    input_tree = strdup(p);
+    i = compare_newick_with_names(input_tree, pars_names, tmpallseqs->n_otu, &differs);
+    if (i != 0) {
+      alert("Names in user-tree and target sequences differ:\n%s in %s only", 
+	       differs, i == 1 ? "user-tree" : "target sequences");
+      tree = NULL;
+    }
+    else {
+      input_tree = (char *)realloc(input_tree, strlen(input_tree) + 4 * tmpallseqs->n_otu + 6 );
+      if( make_binary_or_unrooted(input_tree) != NULL) return NULL;
+      strcat(input_tree, ";");
+      tree = parsimony(pars_seqs, pars_names, tmpallseqs->n_otu, &steps, input_tree, maxbest, NULL, view->protein, no_gui);
+      free(input_tree);
+      if(tree == NULL) {
+	alert("Error in user-tree");
+      }
+      else {
+	final_tree = (char *)malloc(strlen(tree) + 150);
+	sprintf(final_tree, "[%s, user-tree, %d steps, %d sites (%d informative)%s]%s",
+		*pprogname, steps, tmpallseqs->clean_len, info, gapsunknown, tree);
+      }
+    }
+  }
+  else {
+    tree = parsimony_with_jumble(pars_seqs, pars_names, tmpallseqs->n_otu, njumbles, view->protein, 
+				 &steps, &count, maxbest, w_count, no_gui);
+    if (tree) {
+      final_tree = (char *)malloc(strlen(tree) + 150);
+      sprintf(final_tree, "[%s, %d best trees%s, %d steps, %d sites (%d informative)%s]%s",
+	      *pprogname, count, count > 1 ? " (strict consensus)" : "", steps, tmpallseqs->clean_len, info, gapsunknown, tree);
+    }
+  }
+way_out:
+  if (tree) free(tree);
+  free_after_view_to_allseq(tmpallseqs);
+  free(pars_seqs);
+  free(pars_names);
+  return final_tree;
+}
+
 
 void parsimony_dialog(SEA_VIEW *view)
 {
-	int steps, info, njumbles, i, count, maxbest, started = 0, use_bootstrap, 
-		nreplicates, replicates_done, user_tree, nogaps; 
-	char *tree = NULL, *final_tree;
+	int njumbles, i, count, maxbest, started = 0, use_bootstrap, 
+		nreplicates, user_tree, nogaps; 
+	char *final_tree;
 	const char *p, *progname;
   
 	tree_build_interrupted = 0;
@@ -1761,78 +1886,15 @@ void parsimony_dialog(SEA_VIEW *view)
 	}
 
 	my_watch_cursor(w);
-	allseq *tmpallseqs = view_to_allseq(view, nogaps);
-	if(w_gapsunknown->value()) gaps_as_unknown_states(tmpallseqs, view->protein);
-	char **pars_seqs = (char **)malloc(tmpallseqs->n_otu * sizeof(char *));
-	char **pars_names = (char **)malloc(tmpallseqs->n_otu * sizeof(char *));
-	for( i = 0; i < tmpallseqs->n_otu; i++) {
-		pars_seqs[i] = tmpallseqs->c_seq[i]->state;
-		pars_names[i] = tmpallseqs->c_seq[i]->name;
-	}
-	info = informative(pars_seqs, tmpallseqs->n_otu);
-	if(view->protein) progname = "Protpars"; 
-	else {
-	  progname = "Dnapars";
-	  if (tmpallseqs->clean_len >= SHRT_MAX) {
-	    fl_alert("Can't compute parsimony with DNA seqs longer than %d.", SHRT_MAX);
-	    goto way_out;
-	    }
-	  }
-	if(use_bootstrap) {
-		tree = parsimony_with_bootstrap(tmpallseqs, pars_seqs, pars_names, tmpallseqs->n_otu, njumbles, 
-				view->protein, &steps, maxbest, nreplicates, w_count, &replicates_done);
-		if (tree) {
-		  final_tree = (char *)malloc(strlen(tree) + 150);
-		  sprintf(final_tree, "[%s, bootstrap with %d replic., %d steps, %d sites (%d informative)%s]%s",
-				progname, replicates_done, steps, tmpallseqs->clean_len, info, gapsunknown, tree);
-		  }
-		}
-	else if(user_tree) {
-		char *differs, *input_tree;
-		p = view->trees[user_tree - 1];
-		while(*p == ' ') p++;
-		if(*p == '[') p = strchr(p, ']')+ 1;
-		while(*p == ' ') p++;
-		input_tree = strdup(p);
-		i = compare_newick_with_names(input_tree, pars_names, tmpallseqs->n_otu, &differs);
-		if (i != 0) {
-			fl_alert("Names in user-tree and target sequences differ:\n%s in %s only", 
-					 differs, i == 1 ? "user-tree" : "target sequences");
-			tree = NULL;
-			}
-		else {
-		  input_tree = (char *)realloc(input_tree, strlen(input_tree) + 4 * tmpallseqs->n_otu + 6 );
-		  if( make_binary_or_unrooted(input_tree) != NULL) return;
-		  strcat(input_tree, ";");
-		  tree = parsimony(pars_seqs, pars_names, tmpallseqs->n_otu, &steps, input_tree, maxbest, NULL, view->protein);
-		  free(input_tree);
-		  if(tree == NULL) {
-			  fl_alert("Error in user-tree");
-			  }
-		  else {
-			  final_tree = (char *)malloc(strlen(tree) + 150);
-			  sprintf(final_tree, "[%s, user-tree, %d steps, %d sites (%d informative)%s]%s",
-					  progname, steps, tmpallseqs->clean_len, info, gapsunknown, tree);
-			  }
-		}
-	}
-	else {
-	  tree = parsimony_with_jumble(pars_seqs, pars_names, tmpallseqs->n_otu, njumbles, view->protein, 
-								 &steps, &count, maxbest, w_count);
-	  if (tree) {
-		final_tree = (char *)malloc(strlen(tree) + 150);
-		sprintf(final_tree, "[%s, %d best trees%s, %d steps, %d sites (%d informative)%s]%s",
-			progname, count, count > 1 ? " (strict consensus)" : "", steps, tmpallseqs->clean_len, info, gapsunknown, tree);
-		}
-	  }
-way_out:
+
+	final_tree = run_parsimony_method(view, nogaps, gapsunknown, 
+				    use_bootstrap && nreplicates > 1 ? nreplicates : 0, njumbles, 
+					  maxbest, user_tree, w_count, 
+					  &progname, false, fl_alert);
+
 	fl_reset_cursor(w);
 	delete w;
-	free_after_view_to_allseq(tmpallseqs);
-	free(pars_seqs);
-	free(pars_names);
-	if (tree != NULL) {
-	  free(tree);
+	if (final_tree != NULL) {
 	  const char *n = extract_filename(view->masename);
 	  char *title = NULL;
 	  if (n) {
@@ -1849,22 +1911,22 @@ way_out:
 
 
 char *parsimony_with_jumble(char **pars_seqs, char **pars_names, int n_otu, int njumbles, int protein, int *psteps,
-	int *pcount, int maxbest, Fl_Widget *w_count)
+	int *pcount, int maxbest, Fl_Widget *w_count, int no_gui)
 {
 	int i, steps, newsteps, count = 0;
 	char *tree, *newtree, *p, *q;
 	if (njumbles > 0) {
-	  w_count->label("Initial sequence order");
+	  if (w_count) w_count->label("Initial sequence order");
 	  }
-	tree = parsimony(pars_seqs, pars_names, n_otu, &steps, NULL, maxbest, NULL, protein);
+	tree = parsimony(pars_seqs, pars_names, n_otu, &steps, NULL, maxbest, NULL, protein, no_gui);
 	if (!tree) return NULL;
 	newsteps = steps;
 	for( i = 1; i <= njumbles; i++) {
 		char tmp[40];
 		sprintf(tmp, "Jumble attempt: %d steps: %d", i, newsteps); 
-		w_count->label(tmp); 
+		if (w_count) w_count->label(tmp); 
 		jumble_fct(pars_seqs, pars_names, n_otu);
-		newtree = parsimony(pars_seqs, pars_names, n_otu, &newsteps, NULL, maxbest, NULL, protein);
+		newtree = parsimony(pars_seqs, pars_names, n_otu, &newsteps, NULL, maxbest, NULL, protein, no_gui);
 		if (!newtree) break;
 		if(newsteps < steps) {
 			steps = newsteps;
@@ -1902,28 +1964,28 @@ char *parsimony_with_jumble(char **pars_seqs, char **pars_names, int n_otu, int
 
 
 char *parsimony_with_bootstrap(allseq *tmpallseqs, char **pars_seqs, char **pars_names, int n_otu, int njumbles, int protein, int *psteps,
-			int maxbest, int replicates, Fl_Box *w_count, int *preplicates_done)
+			int maxbest, int replicates, Fl_Box *w_count, int *preplicates_done, int no_gui)
 {
 	int i, j, steps, newsteps, count = 0, replicates_done = 0;
 	char *tree, *newtree, *p;
 	
-	w_count->label("Full sequence tree"); 
+	if (w_count) w_count->label("Full sequence tree"); 
 	tree = parsimony_with_jumble(pars_seqs, pars_names, n_otu, njumbles, protein, 
-								 &steps, &count, maxbest, w_count);
+								 &steps, &count, maxbest, w_count, no_gui);
 	if (!tree) return NULL;
 	*psteps = steps;
 	init_add_bootstrap(tree);
 	for(j = 0; j < replicates; j++) {
 		char tmp[30];
 		sprintf(tmp, "Bootstrap replicate: %d", j + 1); 
-		w_count->label(tmp); 
+		if (w_count) w_count->label(tmp); 
 		if (tree_build_interrupted) break;
 		bootstrap_weights(tmpallseqs, FALSE);
-		tree = parsimony(pars_seqs, pars_names, n_otu, &steps, NULL, maxbest, tmpallseqs->wght, protein);
+		tree = parsimony(pars_seqs, pars_names, n_otu, &steps, NULL, maxbest, tmpallseqs->wght, protein, no_gui);
 		if (!tree) break;
 		for( i = 1; i <= njumbles; i++) {
 			jumble_fct(pars_seqs, pars_names, n_otu);
-			newtree = parsimony(pars_seqs, pars_names, n_otu, &newsteps, NULL, maxbest, tmpallseqs->wght, protein);
+			newtree = parsimony(pars_seqs, pars_names, n_otu, &newsteps, NULL, maxbest, tmpallseqs->wght, protein, no_gui);
 			if (!newtree) break;
 			if(newsteps < steps) {
 				steps = newsteps;
@@ -1979,25 +2041,32 @@ static void parsimony_thread(pars_struct* args)
   Fl::unlock();
 }
 
-char *parsimony(char** seq, char** seqname, int notu, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights, bool protein)
+char *parsimony(char** seq, char** seqname, int notu, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights, bool protein, int no_gui)
 {
-  pars_struct *args = (pars_struct*)malloc(sizeof(pars_struct));
-  args->seq = seq;
-  args->seqname = seqname;
-  args->notu = notu;
-  args->steps = steps;
-  args->toevaluate = toevaluate;
-  args->arg_maxtrees = arg_maxtrees;
-  args->bt_weights = bt_weights;
-  args->result = NULL;
-  args->protein = protein;
-  args->done = false;
+  char *p;
   tree_build_interrupted = 0;
-  if ( ! fl_create_thread((void (*) (void *))parsimony_thread, args, 1024) ) {
-    while (!args->done) Fl::wait();
+  if (no_gui) {
+    if (protein) p = protpars(seq, seqname, notu, steps, toevaluate, arg_maxtrees, bt_weights);
+    else  p = dnapars(seq, seqname, notu, steps, toevaluate, arg_maxtrees, bt_weights);
+  }
+  else {
+    pars_struct *args = (pars_struct*)malloc(sizeof(pars_struct));
+    args->seq = seq;
+    args->seqname = seqname;
+    args->notu = notu;
+    args->steps = steps;
+    args->toevaluate = toevaluate;
+    args->arg_maxtrees = arg_maxtrees;
+    args->bt_weights = bt_weights;
+    args->result = NULL;
+    args->protein = protein;
+    args->done = false;
+    if ( ! fl_create_thread((void (*) (void *))parsimony_thread, args, 1024) ) {
+      while (!args->done) Fl::wait();
+    }
+    p = args->result;
+    free(args);
   }
-  char *p = args->result;
-  free(args);
   if (tree_build_interrupted && p) {
     free(p);
     p = NULL;
@@ -2033,7 +2102,7 @@ void free_vector(double *v, long nl, long nh)
 
 int ludcmp(double **a, int n, int *indx, double *d)
 {
-	int i,imax,j,k;
+	int i,imax=0,j,k;
 	double big,dum,sum,temp;
 	double *vv;
 	
@@ -2233,3 +2302,76 @@ char *replace_tree_names_by_rank(const char *tree, char **names, int notu)
 	*q = 0;
 	return newtree;
 }
+
+
+void command_line_phylogeny(int argc, char **argv)
+{
+  char *tree, *p;
+  int replicates = 0, jumbles = 0, i;
+  bool err = false;
+  if (!isarg(argc, argv, "-distance") && !isarg(argc, argv, "-parsimony")) {
+    fputs("Option -build_tree requires either -distance or -parsimony\n", stderr);
+    exit(1);
+    }
+  if (isarg(argc, argv, "-distance") && !(isarg(argc, argv, "-o") || isarg(argc, argv, "-distance_matrix"))) err = true;
+  if (isarg(argc, argv, "-parsimony") && !isarg(argc, argv, "-o")) err = true;
+  if (err) {
+    fputs("Missing '-o fname' option\n", stderr);
+    exit(1);
+    }
+  SEA_VIEW *view = cmdline_read_input_alignment(argc, argv);
+  if ( (p = argname(argc, argv, "-sites")) ) {
+    list_regions *elt = view->regions;
+    while (elt) {
+      if (strcmp(elt->element->name, p) == 0) break;
+      elt = elt->next;
+    }
+    if (elt) view->active_region = elt->element;
+  }
+  
+  if ( (p = argname(argc, argv, "-species")) ) {
+    for (i = 0; i < view->numb_species_sets; i++) {
+      if (strcmp(view->name_species_sets[i], p) == 0) break;
+    }
+    if (i < view->numb_species_sets) {
+      view->sel_seqs = view->list_species_sets[i];
+      view->tot_sel_seqs = 0;
+      for (i = 0; i < view->tot_seqs; i++) if (view->sel_seqs[i]) view->tot_sel_seqs++;
+    }
+  }
+  
+  replicates = argval(argc, argv, "-replicates", 0);
+  if (isarg(argc, argv, "-distance")) { // distance method
+    int distkind = 0; // == observed_p/ndist
+    char *distname;
+    if ( (distname = argname(argc, argv, "-distance"))) {
+      if (strcmp(distname, "JC") == 0) distkind = Jukes_Cantor;
+      else if (strcmp(distname, "K2P") == 0) distkind = Kimura2P;
+      else if (strcmp(distname, "HKY") == 0) distkind = HKY;
+      else if (strcmp(distname, "logdet") == 0) distkind = LogDet;
+      else if (strcmp(distname, "Ka") == 0) distkind = Ka;
+      else if (strcmp(distname, "Ks") == 0) distkind = Ks;
+      else if (strcmp(distname, "Poisson") == 0) distkind = Poisson_pdist;
+      else if (strcmp(distname, "Kimura") == 0) distkind = Kimura_pdist;
+      }
+    if (distkind == 0) distname = strdup("Observed");
+    view->tot_trees = 0;
+    tree = run_distance_method(view, distkind, isarg(argc, argv, "-nogaps"), !isarg(argc, argv, "-NJ"), replicates > 1, replicates, NULL,
+			       distname, distkind == Ka || distkind == Ks, argname(argc, argv, "-distance_matrix"), NULL, true, Fl::fatal);
+    if (isarg(argc, argv, "-distance_matrix")) exit(0);
+  }
+  if (isarg(argc, argv, "-parsimony")) { // parsimony method
+    const char *progname;
+    jumbles = argval(argc, argv, "-jumbles", 0);
+    tree = run_parsimony_method(view, isarg(argc, argv, "-nogaps"), 
+				isarg(argc, argv, "-gaps_as_unknown") ? ", gaps treated as ?" : "", 
+				replicates, jumbles, 10000, 0, NULL, &progname, true, Fl::fatal);
+  }
+  if (tree) {
+    char *outfname = argname(argc, argv, "-o");
+    FILE *out = (strcmp(outfname, "-") != 0 ? fopen(outfname, "w") : stdout);
+    fwrite(tree, strlen(tree), 1, out);
+    fputs("\n", out); fflush(out);
+    if (out != stdout) fclose(out);
+  }
+}
