@@ -14,7 +14,7 @@ void unrooted_zoom_cb(Fl_Widget *o, void *data);
 void frame_and_draw_unrooted(FD_unrooted *fd_unrooted);
 void draw_tree(FD_unrooted *ob);
 int debut_arbre(const char *tree, FD_unrooted *fd_unrooted);
-void my_draw_text(int align, int x, int y, int size, int font, char *chaine);
+void my_draw_text(Fl_Align align, int x, int y, int size, int font, char *chaine);
 hit draw_line(unrooted_branch *br, FD_unrooted *ob, int doit, int char_width,
         int char_height, int descend, int ascent);
 hit try_angle_draw(float w, int *px, int *py, float *pdegrees /* clockwise degrees */, 
@@ -29,8 +29,6 @@ void unrooted_search(FD_unrooted *fd_unrooted, const char *select);
 const char *preptree_unrooted(const char *tree, FD_unrooted *data);
 static void loadphylip(char *arbre, FD_unrooted *data);
 static struct noeud *unrootedset(char *arbre, char *deb, char *fin, branche_noeud **p_int_br, FD_unrooted *data);
-static void parcourir_branches(struct noeud *centre, struct noeud *origine);
-static void process_branche(struct noeud *cote1, struct noeud *cote2, double length);
 double calc_echelle(FD_unrooted *data, int larg);
 bignoeud *cre_new_tree(struct noeud *debut, struct noeud *parent, bignoeud *bigparent);
 void remove_big_root(bignoeud *bigracine);
@@ -63,7 +61,7 @@ extern void *check_alloc(int nbrelt, int sizelt);
 extern void my_watch_cursor(Fl_Window *win);
 extern void fl_reset_cursor(Fl_Window *win);
 extern char *ecrit_arbre_parenth_unrooted(FD_nj_plot *fd_nj_plot, struct noeud *root);
-extern double place_midpoint_root(struct noeud *from, struct noeud *racine);
+extern double place_midpoint_root(struct noeud *from, struct noeud *racine, int fixed_outgroup);
 extern const char *make_binary_or_unrooted(char *arbre);
 extern int make_binary(char *arbre, char *debut, char *fin, int go_down);
 extern char *nextpar(char *pospar);
@@ -411,7 +409,9 @@ void draw_tree(FD_unrooted *ob)
 			encore = chg_phys(ob, &hits);
 		}
 		while (encore);
-		for(num = 0; num < dernier; num++) ob->branches[num].color = FALSE;
+		if (Fl_Surface_Device::surface() == Fl_Display_Device::display_device()) {
+		  for(num = 0; num < dernier; num++) ob->branches[num].color = FALSE;
+		}
 	}
 	for(num=0; num< dernier; num++) {
 		draw_line(ob->branches+num, ob, TRUE, char_width, char_height, 
@@ -549,7 +549,7 @@ return 0;
 }
 
 
-void my_draw_text(int align, int x, int y, int size, int current_font, char *chaine)
+void my_draw_text(Fl_Align align, int x, int y, int size, int current_font, char *chaine)
 {
   fl_font(current_font, size);
   if(align == FL_ALIGN_CENTER) x -= (int)(fl_width(chaine)/2);
@@ -775,7 +775,7 @@ const char *preptree_unrooted(const char *tree, FD_unrooted *data)
 	if(!data->rooted) {
 		data->racine = *(data->tabtax+(++num_noeud));
 		if(num_noeud >= 2*data->notu - 1) return("Error: incorrect tree file");
-		data->root_br_l = place_midpoint_root(data->tabtax[0], data->racine);
+		data->root_br_l = place_midpoint_root(data->tabtax[0], data->racine, FALSE);
 	}
 	else	{
 		return ("Unexpected rooted tree.");
