@@ -65,6 +65,10 @@ static void color_pdf_display(SEA_VIEW *view, int (*calc_color_function)( int ),
 			      int widnames, double x, double y, int fontsize, double char_width, double descender,int num, int current);
 static int calc_vary_lines(int *vary_pos, int widpos);
 static void out_vary_pos(int *vary_pos, int widnames, int widpos, int nl, FILE *textfile, double x, double y);
+SEA_VIEW* read_alignment_file(const char *infile);
+SEA_VIEW *cmdline_read_input_alignment(int argc, char **argv);
+void format_conversion(int argc, char **argv);
+char *process_output_options(int argc, char **argv, known_format& out_format, bool& std_output);
 
 
 /* external */
@@ -87,6 +91,19 @@ extern int save_nexus_file(const char *fname, int ntaxa, int protein,
 	region *region_used, int *sel_seqs, int tot_sel_seqs, int *eachlength,
 	int tot_trees, char **trees, const Fl_Menu_Item *items);
 extern char *my_fgets(char *s, int n, FILE *f);
+extern char *argname(int argc, char *argv[], const char *arg);
+extern int isarg(int argc, char *argv[], const char *arg);
+extern char *create_tmp_filename(void);
+extern void delete_tmp_filename(const char *base_fname);
+extern char *translate_with_gaps(char *seq, int gc);
+extern int get_ncbi_gc_from_comment(char *comment);
+extern int create_gblocks_mask(SEA_VIEW *view, region *myregion, int no_gui, int b5_val, int b4_val, int b3_val, int b2_val);
+extern char *back_translate_with_gaps(char *prot, char *dna);
+extern void del_gap_only_sites(SEA_VIEW *view);
+extern "C" {
+  int get_acnuc_gc_number(int ncbi_gc);
+  int get_ncbi_gc_number(int ncbi_gc);
+}
 
 
 int read_mase_seqs_header(const char *masefname, char ***pseq, char ***pseqname, 
@@ -425,9 +442,10 @@ int read_phylip_align(const char *fname, char ***pseq, char ***pseqname,
 		char ***pcomments, char **pheader, char **err_message)
 {
 FILE *in;
-char *p, *q, c;
+char *p, *q;
+int c;
 static char line[300];
-char **seq, **comments, **seqname;
+char **seq=0, **comments=0, **seqname=0;
 int totseqs, lenseqs, i, l;
 static char ret_message[200];
 *ret_message = 0;
@@ -474,7 +492,7 @@ for(i=0; i<totseqs; i++) {
 	if(c == '\n' || c == '\r') break;
 	if(c != ' ') {
 	  if(q - seq[i] >= lenseqs) goto badfile;
-	  *(q++) = c;
+	  *(q++) = (char)c;
 	  }
 	}
   if(c == '\r') {c = fgetc(in); if(c != '\n') ungetc(c, in); }
@@ -492,7 +510,7 @@ while( l < lenseqs) {
 	  if(c == '\n' || c == '\r') break;
 	  if(c != ' ') {
 		if(q - seq[i] >= lenseqs) goto badfile;
-		*(q++) = c;
+		*(q++) = (char)c;
 		}
 	  }
 	if(c == '\r') {c = fgetc(in); if(c != '\n') ungetc(c, in); }
@@ -880,7 +898,8 @@ return ecrit;
 
 void save_regions(list_regions *regions, FILE *out)
 {
-int total, l_line;
+  int total;
+  unsigned l_line;
 list_segments *segment;
 char line[80];
 do	{
@@ -1300,7 +1319,7 @@ rend NULL si ok
 	un message d'erreur sinon.
 */
 {
-int err;
+int err=0;
 static char err_message[200];
 
 if(format == MASE_FORMAT) 
@@ -1501,7 +1520,7 @@ char line[100], *p;
 int format = -1;
 int nseq, lseq;
 
-in = fopen(filename, "r");
+in = fl_fopen(filename, "r");
 if(in == NULL) return (known_format)-1;
 p = fgets(line, sizeof(line), in);
 if( p == NULL) { fclose(in); return (known_format)-1; }
@@ -1934,3 +1953,345 @@ static void out_vary_pos(int *vary_pos, int widnames, int widpos, int nl, FILE *
   }
 }
 
+
+SEA_VIEW* read_alignment_file(const char *infile )
+{
+  char *err_message;
+  int i;
+  SEA_VIEW* view = new SEA_VIEW;
+  memset(view, 0, sizeof(SEA_VIEW));
+  view->menubar = new Fl_Menu_Bar(0,0, 30, 30);
+  view->menu_trees = new vlength_menu(view->menubar, "Trees", NULL, 0);
+  
+  if (fl_access(infile, R_OK)) {
+    fprintf(stderr, "File %s\nis not readable or does not exist", infile);
+    return NULL;
+    }
+  known_format file_format = what_format(infile);
+  if (file_format < 0) {
+    fprintf(stderr, "File %s\nis not of a format readable by seaview", infile);
+    return NULL;
+  }
+  if (file_format == MASE_FORMAT) {
+    view->tot_seqs = read_mase_seqs_header(infile, &view->sequence,
+					   &view->seqname, &view->comments, &view->header,
+					   &err_message);
+    /* interpreter les regions du header du fichier mase */
+    view->regions = parse_regions_from_header(view->header);
+    /* interpreter les species sets du fichier mase */
+    view->numb_species_sets = parse_species_sets_from_header(view->header,
+							     view->list_species_sets, view->name_species_sets, view->tot_seqs);
+    /* interpreter les trees du fichier mase */
+    parse_trees_from_header(view->header, view);
+    /* interpreter les comment lines du header */
+    view->tot_comment_lines = parse_comment_lines_from_header(view->header,
+							      &(view->comment_line), &(view->comment_name), 
+							      &(view->comment_length) , &(view->max_seq_length));
+  }
+  else if(file_format == FASTA_FORMAT)
+    view->tot_seqs = read_fasta_align(infile, &view->sequence,
+				      &view->seqname, &view->comments, &view->header, &err_message, view->spaces_in_fasta_names);
+  else if(file_format == PHYLIP_FORMAT)
+    view->tot_seqs = read_phylip_align(infile, &view->sequence,
+				       &view->seqname, &view->comments, &view->header, &err_message);
+  else if(file_format == CLUSTAL_FORMAT)
+    view->tot_seqs = read_clustal_align(infile, &view->sequence,
+					&view->seqname, &view->comments, &view->header, &err_message);
+  else if(file_format == MSF_FORMAT)
+    view->tot_seqs = read_msf_align(infile, &view->sequence,
+				    &view->seqname, &view->comments, &view->header, &err_message);
+  else if(file_format == NEXUS_FORMAT) {
+    int **list_sp = NULL; char **name_sp = NULL; int i;
+    view->tot_seqs = read_nexus_align((char*)infile, &view->sequence,
+				      &view->seqname, &view->comments, &view->header,
+				      &err_message, &view->regions, &view->numb_species_sets,
+				      &list_sp, &name_sp, &view->tot_comment_lines, 
+				      &view->comment_name, &view->comment_line, 
+				      &view->comment_length, &view->protein, view);
+    for(i= 0; i < view->numb_species_sets; i++) {
+      view->list_species_sets[i] = list_sp[i];
+      view->name_species_sets[i] = name_sp[i];
+    }
+    if(list_sp != NULL) free(list_sp); 
+    if(name_sp != NULL) free(name_sp);
+  }
+  if (view->tot_seqs == 0) return NULL;
+  if (file_format != NEXUS_FORMAT) view->protein = is_a_protein_seq(view->sequence[0]);
+  view->each_length = new int[view->tot_seqs];
+  for (i = 0; i < view->tot_seqs; i++) {
+    view->each_length[i] = strlen(view->sequence[i]);
+  }
+  view->seq_length = 0;
+  for (i = 0; i < view->tot_seqs; i++) {
+    if (view->each_length[i] > view->seq_length) view->seq_length = view->each_length[i];
+  }
+  view->masename = strdup(infile);
+  view->format_for_save = file_format;
+  return view;
+}
+
+SEA_VIEW *cmdline_read_input_alignment(int argc, char **argv)
+{
+  char *fname, line[500];
+  fname = argv[argc-1];
+  if (strcmp(fname, "-") == 0) {
+    fname = create_tmp_filename();
+    FILE *out = fopen(fname, "w");
+    while (fgets(line, sizeof(line), stdin) != NULL) {
+      fputs(line, out);
+    }
+    fclose(out);
+  }
+  SEA_VIEW *view = read_alignment_file(fname);
+  if (strcmp(argv[argc-1], "-") == 0) delete_tmp_filename(fname);
+  if (!view) {
+    fputs("\n", stderr);
+    exit(1);
+  }
+  return view;
+}
+
+void format_conversion(int argc, char **argv)
+{
+  char *err_message, *p;
+  int i, j, gc;
+  bool std_output = false;
+  bool save_fragment = false;
+  region *myregion;
+  
+  SEA_VIEW *view = cmdline_read_input_alignment(argc, argv);
+  if (view == NULL) {
+    fprintf(stderr, "No sequence found in %s", argv[argc-1]);
+    exit(1);
+  }  
+  known_format out_format = view->format_for_save;
+  char *outfile = process_output_options(argc, argv, out_format, std_output);
+  
+  if ( !view->protein && isarg(argc, argv, "-translate")) {
+    bool no_terminal_stop = isarg(argc, argv, "-no_terminal_stop");
+    for (i = 0; i < view->tot_seqs; i++) {
+      gc = (view->comments != NULL ? get_ncbi_gc_from_comment(view->comments[i]) : 1);
+      p = translate_with_gaps(view->sequence[i], get_acnuc_gc_number(gc));
+      free(view->sequence[i]);
+      view->sequence[i] = p;
+      if (no_terminal_stop) {
+	char *q = p + strlen(p) - 1;
+	while (q > p && *q == '-') q--;
+	if (q >= p && *q == '*') *q = '-';
+      }
+      view->each_length[i] = strlen(p);
+      view->regions = NULL;
+      }
+    view->protein = true;
+    }
+  
+  while ( (p = argname(argc, argv, "-def_species_group")) != NULL) {
+    p = strtok(p, ",");
+    view->name_species_sets[view->numb_species_sets] = strdup(p);
+    view->list_species_sets[view->numb_species_sets] = (int*)calloc(view->tot_seqs, sizeof(int));
+    while (true) {
+      p = strtok(NULL, ",");
+      if (!p) break;
+      if (strchr(p, '-') == NULL) {
+	sscanf(p, "%d", &i);
+	view->list_species_sets[view->numb_species_sets][i-1] = 1;
+      }
+      else {
+	sscanf(p, "%d-%d", &i, &j);
+	if (i <= j) {
+	  while (i <= j) view->list_species_sets[view->numb_species_sets][i++ - 1] = 1;
+	  }
+      }
+    }
+    view->numb_species_sets++;
+    for (i = 0; i < argc; i++) {
+      if (strcmp(argv[i], "-def_species_group") == 0) {
+	*argv[i] = '+';
+	break;
+      }
+    }
+  }
+  
+  while ( (p = argname(argc, argv, "-def_site_selection")) != NULL) {
+    int from, to;
+    list_segments *current_seg;
+    region *site_selection = (region*)calloc(1, sizeof(region));
+    list_regions *last_reg = (list_regions*)calloc(1, sizeof(list_regions));
+    last_reg->element = site_selection;
+    if (!view->regions) {
+      view->regions = last_reg;
+      }
+    else {
+      list_regions *current_reg;
+      current_reg = view->regions;
+      while (current_reg->next) current_reg = current_reg->next;
+      current_reg->next = last_reg;
+      }
+    p = strtok(p, ",");
+    site_selection->name = strdup(p);
+    while (true) {
+      p = strtok(NULL, ",");
+      if (!p) break;
+      if (strchr(p, '-') == NULL) {
+	sscanf(p, "%d", &from);
+	to = from;      }
+      else {
+	sscanf(p, "%d-%d", &from, &to);
+	if (from > to) continue;
+      }
+      if (to > view->seq_length) to = view->seq_length;
+      if (from > view->seq_length) continue;
+      list_segments *seg = (list_segments*)malloc(sizeof(list_segments));
+      seg->debut = from;
+      seg->fin = to;
+      seg->next = NULL;
+      if (!site_selection->list) {
+	site_selection->list = seg;
+	}
+      else {
+	current_seg->next = seg;
+	}
+      current_seg = seg;
+    }
+    for (i = 0; i < argc; i++) {
+      if (strcmp(argv[i], "-def_site_selection") == 0) {
+	*argv[i] = '+';
+	break;
+      }
+    }
+  }  
+
+  if (isarg(argc, argv, "-gblocks")) { // gblocks method
+    myregion = (region*)malloc(sizeof(region));
+    myregion->name = strdup("Gblocks");
+    view->region_line = (char*)malloc(view->seq_length + 1);
+    create_gblocks_mask(view, myregion, true, isarg(argc, argv, "-b5"), isarg(argc, argv, "-b4"), 
+			isarg(argc, argv, "-b3"), isarg(argc, argv, "-b2"));
+    list_regions *r = view->regions, *pre = NULL;
+    while (r) {
+      if (strcmp(r->element->name, myregion->name) == 0) break;
+      pre = r;
+      r = r->next;
+    }
+    if (r) {
+      free(r->element->name);
+      r->element = myregion;
+    }
+    else {
+      list_regions *elt = (list_regions*)malloc(sizeof(list_regions));
+      elt->element = myregion;
+      elt->next = NULL;
+      if (pre) pre->next = elt;
+      else view->regions = elt;
+    }
+  }
+  
+  if ( (p = argname(argc, argv, "-sites")) ) {
+    save_fragment = true;
+    list_regions *elt = view->regions;
+    while (elt) {
+      if (strcmp(elt->element->name, p) == 0) break;
+      elt = elt->next;
+    }
+    if (elt) view->active_region = elt->element;
+  }
+  
+  if ( (p = argname(argc, argv, "-species")) ) {
+    save_fragment = true;
+    for (i = 0; i < view->numb_species_sets; i++) {
+      if (strcmp(view->name_species_sets[i], p) == 0) break;
+    }
+    if (i < view->numb_species_sets) {
+      view->sel_seqs = view->list_species_sets[i];
+      view->tot_sel_seqs = 0;
+      for (i = 0; i < view->tot_seqs; i++) if (view->sel_seqs[i]) view->tot_sel_seqs++;
+    }
+  }
+  if (isarg(argc, argv, "-del_gap_only_sites") && !isarg(argc, argv, "-sites")) del_gap_only_sites(view);
+  
+  if (save_fragment) {
+    myregion = view->active_region;
+    if (myregion == NULL) {
+      myregion = (region *)malloc(sizeof(region));
+      myregion->list = (list_segments *)malloc(sizeof(list_segments));
+      myregion->list->debut = 1;
+      myregion->list->fin = view->seq_length;
+      myregion->list->next = NULL;
+      myregion->name = (char *)"all";
+    }
+    err_message = save_alignment_or_region(outfile, view->sequence, view->comments,
+					   view->header, view->seqname, view->tot_seqs, view->each_length,
+					   NULL, myregion, out_format,
+					   0, NULL, NULL, view->sel_seqs, view->tot_sel_seqs, 
+					   view->protein, 0, NULL, NULL, view->phylipwidnames,
+					   view->tot_trees, view->trees, 
+					   view->menu_trees->vitem(0), view->spaces_in_fasta_names);
+  }
+  else {
+    err_message = save_alignment_or_region(outfile, view->sequence, view->comments,
+				 view->header, view->seqname, view->tot_seqs, view->each_length,
+				 view->regions, NULL, out_format,
+				 view->numb_species_sets, view->list_species_sets,
+				 view->name_species_sets, NULL, 0, view->protein,
+				 view->tot_comment_lines, view->comment_name, 
+				 view->comment_line, view->phylipwidnames,
+				 view->tot_trees, view->trees, 
+				 view->menu_trees->vitem(0), view->spaces_in_fasta_names);
+  }
+  if (err_message) {
+    fprintf(stderr, "%s\n", err_message);
+    if (std_output) delete_tmp_filename(outfile);
+    exit(1);
+    }
+  if (std_output) {
+    char line[200];
+    FILE *in = fopen(outfile, "r");
+    while (fgets(line, sizeof(line), in) != NULL) {
+      fputs(line, stdout);
+    }
+    fclose(in);
+    fflush(stdout);
+    delete_tmp_filename(outfile);
+    }
+  exit(0);
+}
+
+
+char *process_output_options(int argc, char **argv, known_format& out_format, bool& std_output)
+{
+  int i;
+  char upper_format[20], *p;
+  std_output = false;
+  char *outformatname = argname(argc, argv, "-output_format");
+  if (outformatname) {
+    majuscules(outformatname);
+    for (i = 0; i < nbr_formats; i++) {
+      strcpy(upper_format, f_format_names[i]);
+      majuscules(upper_format);
+      if (strcmp(upper_format, outformatname)== 0) break;
+    }
+    if (i >= nbr_formats) {
+      fprintf(stderr, "'%s' is not of a format known by seaview", outformatname);
+      exit(1);
+    }
+    out_format = (known_format)i;
+  }
+  char *outfile = argname(argc, argv, "-o");
+  if (outfile == NULL) {
+    outfile = new char[strlen(argv[argc-1]) + 12];
+    strcpy((char*)outfile, argv[argc-1]);
+    p = strrchr(outfile, '.');
+    if (!p) { p = (char*)outfile + strlen(outfile); *p = '.'; }
+    strcpy(p+1, f_format_exts[out_format]);
+    if (strcmp(outfile, argv[argc-1]) == 0) {
+      strcpy(p, "_2.");
+      strcpy(p+3, f_format_exts[out_format]);
+    }
+  }
+  else if (strcmp(outfile, "-") == 0) {
+    outfile = create_tmp_filename();
+    std_output = true;
+  }
+  return outfile;
+}
+
