@@ -50,14 +50,14 @@ void ps_draw_line(int x1, int y1, int x2, int y2, FD_matpt *matpt);
 void ps_set_mat_data(FD_matpt *matpt);
 void plot_button_proc(Fl_Widget *ob, void *data);
 int really_close(FD_matpt *matpt);
-void exit_button_proc(Fl_Widget *ob, void *data);
+void exit_button_proc(Fl_Widget *ob, FD_matpt *data);
 void move_mat_proc(Fl_Widget *ob, void *data);
 void move_box_step(Fl_Widget *ob, void *extra);
 void region_size_proc(Fl_Widget *ob, void *data);
 char *readseq(FILE *infile, int *slong, char **seqname);
 char *readseqfile(char *fname, int *seqlong1, int *seqlong2,
 	char **name1, char **name2, char **seq1, char **seq2);
-int compute_diags(FD_matpt *matpt, int *erreur, char **errmess);
+void compute_diags(FD_matpt *matpt);
 void compute_title(FD_matpt *matpt);
 int find_next_gap_site(int pos, gap_site *gap_sites, int tot_gap_sites);
 void new_gap_site(int pos, int l, gap_site *gap_sites, int *p_tot_gap_sites,
@@ -75,6 +75,7 @@ void record_alignment_callback(Fl_Widget *ob, void *data);
 extern int insert_gaps_at(SEA_VIEW *view, int seq, int site, int total);
 extern void update_current_seq_length(int newlength, SEA_VIEW *view);
 void update_col_lines(int num, SEA_VIEW *view);
+void matpt_idle_cb(FD_matpt *matpt);
 
 class matpt_panel : public Fl_Widget {
     FL_EXPORT void draw(void);
@@ -109,7 +110,7 @@ SEA_VIEW *view = (SEA_VIEW *)seaview_data;
 if(first) {
 	first = FALSE;
    	matpt = create_form_matpt();
-	matpt->form->callback(exit_button_proc, matpt);
+	matpt->form->callback((Fl_Callback*)exit_button_proc, matpt);
 	matpt->seq1 = matpt->seq2 = NULL;
 	matpt->seaview_data = seaview_data;
 	old_seq1 = NULL; old_seq2 = NULL;
@@ -153,7 +154,6 @@ obj->labelsize(si);
 return obj;
 }
 
-
 FD_matpt *create_form_matpt(void)
 {
 Fl_Widget *obj;
@@ -214,7 +214,7 @@ fin += 2;
 Fl_Widget *close = cre_button(fin, curr_y, &width, but_height, fontsize, "Close");
 ((Fl_Button *)close)->shortcut(FL_COMMAND | 'w');
 fin += width;
-close->callback(exit_button_proc, fdui);
+close->callback((Fl_Callback*)exit_button_proc, fdui);
 #endif
 
 width = fin - curr_x;
@@ -568,7 +568,10 @@ log_to_phys(matpt->hitx + matpt->region_size, matpt->hity + matpt->region_size,
 	&pxf, &pyf, matpt);
 // rectangle mis pour avoir h > 0
 old_rect.x = px; old_rect.y = pyf; old_rect.w = pxf - px; old_rect.h = py - pyf;
-fl_overlay_rect(old_rect.x, old_rect.y, old_rect.w, old_rect.h);
+  if (old_rect.x + old_rect.w > 0 && old_rect.y + old_rect.h > 0 && 
+      old_rect.y <= matpt->form->h() && old_rect.x <= matpt->form->w()) {
+      fl_overlay_rect(old_rect.x, old_rect.y, old_rect.w, old_rect.h);
+    }
 fl_color(FL_BLACK);
 matpt->rect_only = FALSE;
 }
@@ -577,8 +580,6 @@ matpt->rect_only = FALSE;
 void matpt_panel::draw(void)
 {
 FD_matpt *matpt = (FD_matpt *)this->user_data();
-static int erreur = FALSE;
-static char *errmess;
 int i, pxd, pyd, pxf, pyf;
 static int old_w = 0, old_h = 0;
 	this->clear_damage(); // tres important!
@@ -622,10 +623,11 @@ static int old_w = 0, old_h = 0;
 	log_to_phys(0, matpt->seqlong2, &pxf, &pyf, matpt);
 	fl_line(pxd, pyd, pxf, pyf);
 	if(matpt->need_compute) {
-		matpt->totsegments = compute_diags(matpt, &erreur, &errmess);
+		compute_diags(matpt);
+		fl_pop_clip();
+		return;
 		}
 	else	{
-		erreur = FALSE;
 		for (i = 0; i <= matpt->totsegments; i++) {
 			log_to_phys(segments[i].x,
 				segments[i].y, 
@@ -641,10 +643,6 @@ label_draw_rect:
 	fl_pop_clip();
 	compute_region_text(matpt);
 	fl_reset_cursor(this->window());
-	if(erreur) {
-		fl_message("%s", errmess);
-		erreur = FALSE;
-		}
 }
 
 
@@ -895,9 +893,11 @@ return TRUE;
 }
 
 
-void exit_button_proc(Fl_Widget *ob, void *data)
+void exit_button_proc(Fl_Widget *ob, FD_matpt *matpt)
 {
-if( really_close( (FD_matpt *)data ) )  ((FD_matpt *)data)->form->hide();
+if ( really_close(matpt) )  {
+  matpt->form->hide();
+  }
 }
 
 
@@ -1176,147 +1176,156 @@ fl_reset_cursor(ob->window());
 }
 
 
+void matpt_idle_cb(FD_matpt *matpt)
+{
+  static char err_seg[]="Only part of the dot plot was computed";
+#ifdef __APPLE__
+  const int step = 100; // on Mac OS, idle callbacks are slow to be called, so better give them more work
+#else
+  const int step = 1;
+#endif
+  int i, deb, fin, nmatch, finw, diff, lfrag, k, pp, d, deb1, fin1, fin2, 
+    deb2, pxd, pxf, pyd, pyf, erreur = FALSE;
+  int kk0 = matpt->kk;
+  if ( !matpt->form->visible() ) {
+    goto label_fin;
+  }
+  matpt->form->make_current();
+  if (matpt->interrupted) {
+    goto label_fin;
+  }
+  fl_push_clip(matpt->mat_panel->x(), matpt->mat_panel->y(), matpt->mat_panel->w(), matpt->mat_panel->h());
+  for (; matpt->kk < kk0 + step && matpt->kk <= matpt->dmax + 1; matpt->kk++) {
+      /* pour faire la boucle k = [dmin,dmax] en partant du centre et s'eloignant */
+      matpt->flip = - matpt->flip;
+      k = matpt->centre + ((matpt->kk - matpt->dmin)/2) * matpt->flip;
+      deb= ( k>=0 ? k+1 : 1);
+      fin= FL_min( matpt->seqlong2_nogaps+k , matpt->seqlong1_nogaps);
+      /*on met it[1] ssi bases egales en majuscu dans les 2 seqs sur la diagonale k*/
+      for(i = deb; i <= fin; i++) {
+	diff = matpt->seq1[matpt->addgaps1[i]] - matpt->seq2[matpt->addgaps2[i-k]];
+	matpt->it[i] = (diff == 0 || diff == 32 || diff == -32 ? 1 : 0);
+      }
+      matpt->it[fin + 1] = 0;
+      /* on met it[1] ssi au moins matpt->identites bases egales dans fenetre de taille fenetre 
+       sur la diagonale k 
+       */
+      pp=matpt->it[deb];
+      nmatch=0;
+      for (i = deb; i < deb+matpt->fenetre; i++) nmatch += matpt->it[i];
+      if(nmatch>=matpt->identites)
+	matpt->it[deb]=1;
+      else
+	matpt->it[deb]=0;
+      finw = fin-matpt->fenetre+1;
+      for(i = deb+1; i <= finw; i++) {
+	nmatch += (matpt->it[i+matpt->fenetre-1]-pp);
+	pp=matpt->it[i];
+	if(nmatch>=matpt->identites)
+	  matpt->it[i]=1;
+	else
+	  matpt->it[i]=0;
+      }
+      i=deb-1;
+      /* calcul de d = debut des runs de 1, i = suivant de la fin du run de 1 */
+      while(i<finw) {
+	i++;
+	if(matpt->it[i]==0) continue;
+	d=i;
+	do	{
+	  i++;
+	  if(i>finw) break;
+	}
+	while(matpt->it[i]==1);
+	/* decoupage du segment sans gaps en autant de morceaux non coupes de gaps */
+	deb1 = matpt->addgaps1[d]; deb2 = matpt->addgaps2[d-k]; lfrag = 0;
+	do	{
+	  fin1 = deb1; fin2 = deb2;
+	  while(lfrag < i - d && matpt->seq1[fin1] != '-' && 
+		matpt->seq2[fin2] != '-') {
+	    fin1++; fin2++; lfrag++;
+	  }
+	  fin1--; fin2--;
+	  if (matpt->totsegments >= MAXSEGMENTS) {
+	    erreur = TRUE;
+	    fl_pop_clip();
+	    goto label_fin;
+	  }
+	  matpt->totsegments++;
+	  segments[matpt->totsegments].x = deb1;
+	  segments[matpt->totsegments].y = deb2;
+	  segments[matpt->totsegments].longueur = fin1 - deb1 + 1;
+	  log_to_phys(segments[matpt->totsegments].x,
+		      segments[matpt->totsegments].y, 
+		      &pxd, &pyd, matpt);
+	  log_to_phys(segments[matpt->totsegments].x + 
+		      segments[matpt->totsegments].longueur, 
+		      segments[matpt->totsegments].y + 
+		      segments[matpt->totsegments].longueur, 
+		      &pxf, &pyf, matpt);
+	  fl_line(pxd, pyd, pxf, pyf);
+	  deb1 = fin1+1; deb2 = fin2+1;
+	  while(matpt->seq1[deb1] == '-') deb1++;
+	  while(matpt->seq2[deb2] == '-') deb2++;
+	}
+	while(lfrag < i - d);
+      }
+  }
+  fl_pop_clip();
+  if (matpt->kk <= matpt->dmax + 1) return;
+label_fin:
+  Fl::remove_idle((Fl_Idle_Handler)matpt_idle_cb, matpt);
+  free(matpt->it);
+  free(matpt->addgaps1); free(matpt->addgaps2);
+  if (matpt->form->visible()) draw_rect(matpt);
+  fl_reset_cursor(matpt->form);
+  if (erreur) {
+    fl_message("%s", err_seg);
+    }
+}
+
 /* dot plot pour sequences avec gaps:
 les segments sont calcules sans les gaps
 leurs coordonnees sont reportees sur les seqs avec gaps
 puis ils sont coupes en autant de morceaux sans gap
 */
-int compute_diags(FD_matpt *matpt, int *erreur, char **errmess)
+void compute_diags(FD_matpt *matpt)
 {
-int lwin,  match, seqlong1, seqlong2, longmax, diff;
-char *seq1, *seq2;
-int pxd, pyd, pxf, pyf;
-int dmin, dmax, k, deb, fin, *it, i, pp, nmatch, finw, d, totsegments, kk,
-	centre, flip;
+int  longmax;
 static char err_mem[]="Not enough memory";
-static char err_seg[]="Only part of the dot plot was computed";
-int *addgaps1, *addgaps2, deb1, deb2, fin1, fin2, lfrag;
 char *p;
 
 matpt->need_compute = FALSE;
-lwin = matpt->fenetre; match = matpt->identites; 
-seqlong1 = matpt->seqlong1; seqlong2 = matpt->seqlong2;
-seq1 = matpt->seq1; seq2 = matpt->seq2;
+  matpt->seqlong1_nogaps = matpt->seqlong1; matpt->seqlong2_nogaps = matpt->seqlong2;
 
-totsegments= -1;
+matpt->totsegments= -1;
 /* calcul corresp coord sans gap -> coord avec gap */
-addgaps1 = (int *)malloc( (seqlong1+2) * sizeof(int));
-addgaps2 = (int *)malloc( (seqlong2+2) * sizeof(int));
-if(addgaps1 == NULL || addgaps2 == NULL) {
-	*erreur = TRUE;
-	*errmess = err_mem;
-	return totsegments;
-	}
-p = seq1; seqlong1 = 0;
+matpt->addgaps1 = (int *)malloc( (matpt->seqlong1_nogaps+2) * sizeof(int));
+matpt->addgaps2 = (int *)malloc( (matpt->seqlong2_nogaps+2) * sizeof(int));
+if(matpt->addgaps1 == NULL || matpt->addgaps2 == NULL) {
+  goto way_out;
+  }
+p = matpt->seq1; matpt->seqlong1_nogaps = 0;
 while( *(++p) != 0) {
 	if( *p == '-') continue;
-	addgaps1[++seqlong1] = p - seq1;
+	matpt->addgaps1[++matpt->seqlong1_nogaps] = p - matpt->seq1;
 	}
-p = seq2; seqlong2 = 0;
+p = matpt->seq2; matpt->seqlong2_nogaps = 0;
 while( *(++p) != 0) {
 	if( *p == '-') continue;
-	addgaps2[++seqlong2] = p - seq2;
+	matpt->addgaps2[++matpt->seqlong2_nogaps] = p - matpt->seq2;
 	}
 
-dmin = lwin-seqlong2; dmax = seqlong1-lwin-1;
-longmax = FL_max(seqlong1, seqlong2);
-it = (int *)malloc( (longmax + 2) * sizeof(int));
-if(it == NULL) {
-	*erreur = TRUE;
-	*errmess = err_mem;
-	return totsegments;
-	}
-centre = (dmin + dmax)/2; flip = 1;
-for(kk = dmin + 1; kk <= dmax + 1; kk++) {
-	if( Fl::ready() ) {
-		Fl::wait(0);
-		if (matpt->interrupted) {
-		  do Fl::wait(0); while (Fl::ready());
-		  break;
-		  }
-		if( !matpt->form->visible() ) {
-			*erreur = TRUE;
-			goto fin;
-			}
-		}
-/* pour faire la boucle k = [dmin,dmax] en partant du centre et s'eloignant */
-	flip = - flip;
-	k = centre + ((kk - dmin)/2) * flip;
-	deb= ( k>=0 ? k+1 : 1);
-	fin= FL_min( seqlong2+k , seqlong1);
-/*on met it[1] ssi bases egales en majuscu dans les 2 seqs sur la diagonale k*/
-	for(i = deb; i <= fin; i++) {
-		diff = seq1[addgaps1[i]] - seq2[addgaps2[i-k]];
-		it[i] = (diff == 0 || diff == 32 || diff == -32 ? 1 : 0);
-		}
-	it[fin + 1] = 0;
-/* on met it[1] ssi au moins match bases egales dans fenetre de taille lwin 
-sur la diagonale k 
-*/
-	pp=it[deb];
-	nmatch=0;
-	for(i = deb; i < deb+lwin; i++) nmatch += it[i];
-	if(nmatch>=match)
-		it[deb]=1;
-	else
-		it[deb]=0;
-	finw = fin-lwin+1;
-	for(i = deb+1; i <= finw; i++) {
-		nmatch += (it[i+lwin-1]-pp);
-		pp=it[i];
-		if(nmatch>=match)
-			it[i]=1;
-		else
-			it[i]=0;
-		}
-	i=deb-1;
-/* calcul de d = debut des runs de 1, i = suivant de la fin du run de 1 */
-	while(i<finw) {
-		i++;
-		if(it[i]==0) continue;
-		d=i;
-		do	{
-			i++;
-			if(i>finw) break;
-			}
-		while(it[i]==1);
-/* decoupage du segment sans gaps en autant de morceaux non coupes de gaps */
-		deb1 = addgaps1[d]; deb2 = addgaps2[d-k]; lfrag = 0;
-		do	{
-			fin1 = deb1; fin2 = deb2;
-			while(lfrag < i - d && seq1[fin1] != '-' && 
-							seq2[fin2] != '-') {
-				fin1++; fin2++; lfrag++;
-				}
-			fin1--; fin2--;
-			if(totsegments>=MAXSEGMENTS) {
-				*erreur = TRUE;
-				*errmess = err_seg;
-				goto fin;
-				}
-			totsegments++;
-			segments[totsegments].x = deb1;
-			segments[totsegments].y = deb2;
-			segments[totsegments].longueur = fin1 - deb1 + 1;
-			log_to_phys(segments[totsegments].x,
-				segments[totsegments].y, 
-				&pxd, &pyd, matpt);
-			log_to_phys(segments[totsegments].x + 
-				segments[totsegments].longueur, 
-				segments[totsegments].y + 
-				segments[totsegments].longueur, 
-				&pxf, &pyf, matpt);
-			fl_line(pxd, pyd, pxf, pyf);
-			deb1 = fin1+1; deb2 = fin2+1;
-			while(seq1[deb1] == '-') deb1++;
-			while(seq2[deb2] == '-') deb2++;
-			}
-		while(lfrag < i - d);
-		}
-	}
-*erreur = FALSE;
-fin:
-free(it);
-free(addgaps1); free(addgaps2);
-return totsegments;
+matpt->dmin = matpt->fenetre-matpt->seqlong2_nogaps; matpt->dmax = matpt->seqlong1_nogaps-matpt->fenetre-1;
+longmax = FL_max(matpt->seqlong1_nogaps, matpt->seqlong2_nogaps);
+matpt->it = (int *)malloc( (longmax + 2) * sizeof(int));
+if(matpt->it == NULL) {
+  goto way_out;
+  }
+matpt->centre = (matpt->dmin + matpt->dmax)/2; matpt->flip = 1;
+matpt->kk = matpt->dmin + 1;
+Fl::add_idle((Fl_Idle_Handler)matpt_idle_cb, matpt);
+return;
+way_out:
+  fl_message("%s", err_mem);
 }
